import textwrap

import platform
import pytest

from conan.test.assets.autotools import gen_makefile
from conan.test.assets.sources import gen_function_cpp, gen_function_h
from conan.test.utils.tools import TestClient
from conan.tools.gnu.makedeps import CONAN_MAKEFILE_FILENAME


@pytest.mark.tool("make" if platform.system() != "Windows" else "msys2")
def test_make_deps_definitions_escape():
    """
    MakeDeps has to escape the definitions properly.
    """
    client = TestClient(path_with_spaces=False)
    conanfile = textwrap.dedent(r'''
        from conan import ConanFile
        class HelloLib(ConanFile):
            def package_info(self):
                self.cpp_info.defines.append("USER_CONFIG=\"user_config.h\"")
                self.cpp_info.defines.append('OTHER="other.h"')
                self.cpp_info.cflags.append("flag1=\"my flag1\"")
                self.cpp_info.cxxflags.append('flag2="my flag2"')
        ''')
    client.save({"conanfile.py": conanfile})
    client.run("export . --name=hello --version=0.1.0")
    client.run("install --requires=hello/0.1.0 --build=missing -g MakeDeps")
    client.run_command(f"make --print-data-base -f {CONAN_MAKEFILE_FILENAME}", assert_error=True)
    assert r'CONAN_CXXFLAGS_HELLO = flag2=\"my flag2\"' in client.out
    assert r'CONAN_CFLAGS_HELLO = flag1=\"my flag1\"' in client.out
    assert r'CONAN_DEFINES_HELLO = $(CONAN_DEFINE_FLAG)USER_CONFIG="user_config.h" $(CONAN_DEFINE_FLAG)OTHER="other.h"' in client.out


def test_makedeps_with_tool_requires():
    """
    MakeDeps has to create any test requires to be declared on the recipe.
    """
    conanfile = textwrap.dedent(r'''
        from conan import ConanFile
        class HelloLib(ConanFile):
            def package_info(self):
                self.cpp_info.libs = [self.name]
        ''')
    client = TestClient(path_with_spaces=False)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=hello --version=0.1.0")
    client.run("create . --name=test --version=0.1.0")
    client.run("create . --name=tool --version=0.1.0")
    # Create library having build and test requires
    conanfile = textwrap.dedent(r'''
        from conan import ConanFile
        class HelloLib(ConanFile):
            def build_requirements(self):
                self.test_requires('hello/0.1.0')
                self.test_requires('test/0.1.0')
                self.tool_requires('tool/0.1.0')
        ''')
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -g MakeDeps")
    content = client.load(CONAN_MAKEFILE_FILENAME)
    assert "CONAN_NAME_TEST" in content
    assert "CONAN_NAME_HELLO" in content
    assert "CONAN_NAME_TOOL" not in content


@pytest.mark.tool("make" if platform.system() != "Windows" else "msys2")
def test_makedeps_with_makefile_build():
    """
    Build a small application using MakeDeps generator and with components
    """
    client = TestClient(path_with_spaces=False)
    with client.chdir("lib"):
        client.save({"Makefile": gen_makefile(libs=["hello"]),
                    "hello.cpp": gen_function_cpp(name="hello", includes=["hello"], calls=["hello"]),
                    "hello.h": gen_function_h(name="hello"),
                    "conanfile.py": textwrap.dedent(r'''
                    from conan import ConanFile
                    from conan.tools.gnu import Autotools
                    from conan.tools.files import copy
                    import os
                    class PackageConan(ConanFile):
                        exports_sources = ("Makefile", "hello.cpp", "hello.h")
                        settings = "os", "arch", "compiler"

                        def configure(self):
                            self.win_bash = self.settings.os == "Windows"

                        def build(self):
                            self.run("make -f Makefile")

                        def package(self):
                            copy(self, "libhello.a", src=self.build_folder, dst=os.path.join(self.package_folder, "lib"))
                            copy(self, "hello.h", src=self.source_folder, dst=os.path.join(self.package_folder, "include"))

                        def package_info(self):
                            self.cpp_info.components["qux"].includedirs = ["include"]
                            self.cpp_info.components["baz"].libs = ["hello"]
                            self.cpp_info.components["baz"].defines = ["FOOBAR=1"]
                    ''')
                     })
        client.run('create . --name=hello --version=0.1.0 -c tools.microsoft.bash:subsystem=msys2 -c tools.microsoft.bash:path=bash')
    with client.chdir("global"):
        # Consume from global variables
        client.run("install --requires=hello/0.1.0 -pr:b=default -pr:h=default -g MakeDeps -of build")
        client.save({"Makefile": textwrap.dedent('''
            include build/conandeps.mk
            CXXFLAGS            += $(CONAN_CXXFLAGS)
            CPPFLAGS            += $(addprefix -I, $(CONAN_INCLUDE_DIRS))
            CPPFLAGS            += $(addprefix -D, $(CONAN_DEFINES))
            LDFLAGS             += $(addprefix -L, $(CONAN_LIB_DIRS))
            LDLIBS              += $(addprefix -l, $(CONAN_LIBS))
            EXELINKFLAGS        += $(CONAN_EXELINKFLAGS)

            all:
            \t$(CXX) main.cpp $(CPPFLAGS) $(CXXFLAGS) $(LDFLAGS) $(LDLIBS) $(EXELINKFLAGS) -o main
            '''),
            "main.cpp": gen_function_cpp(name="main", includes=["hello"], calls=["hello"]),
            })
        client.run_command("make -f Makefile")
    with client.chdir("components"):
        # Consume from components
        client.run("install --requires=hello/0.1.0 -pr:b=default -pr:h=default -g MakeDeps -of build")
        client.save({"Makefile": textwrap.dedent('''
            include build/conandeps.mk
            CXXFLAGS            += $(CONAN_CXXFLAGS)
            CPPFLAGS            += $(addprefix -I, $(CONAN_INCLUDE_DIRS_HELLO_QUX))
            CPPFLAGS            += $(addprefix -D, $(CONAN_DEFINES) $(CONAN_DEFINES_HELLO_BAZ))
            LDFLAGS             += $(addprefix -L, $(CONAN_LIB_DIRS_HELLO_BAZ))
            LDLIBS              += $(addprefix -l, $(CONAN_LIBS_HELLO_BAZ))
            EXELINKFLAGS        += $(CONAN_EXELINKFLAGS)

            all:
            \t$(CXX) main.cpp $(CPPFLAGS) $(CXXFLAGS) $(LDFLAGS) $(LDLIBS) $(EXELINKFLAGS) -o main
            '''),
            "main.cpp": gen_function_cpp(name="main", includes=["hello"], calls=["hello"]),
            })
        client.run_command("make -f Makefile")



import os
import platform
import textwrap

import pytest

from conan.test.assets.sources import gen_function_cpp, gen_function_h
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for Darwin")
@pytest.mark.tool("bazel", "6.x")  # not working for Bazel 7.x
def test_bazel_simple_cross_compilation():
    profile = textwrap.dedent("""
    [settings]
    arch=x86_64
    build_type=Release
    compiler=apple-clang
    compiler.cppstd=gnu17
    compiler.libcxx=libc++
    compiler.version=13.0
    os=Macos
    """)
    profile_host = textwrap.dedent("""
    [settings]
    arch=armv8
    build_type=Release
    compiler=apple-clang
    compiler.cppstd=gnu17
    compiler.libcxx=libc++
    compiler.version=13.0
    os=Macos
    """)
    conanfile = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.google import Bazel, bazel_layout

    class MyappConan(ConanFile):
        name = "myapp"
        version = "1.0"
        settings = "os", "compiler", "build_type", "arch"
        generators = "BazelToolchain"

        def config_options(self):
            if self.settings.os == "Windows":
                del self.options.fPIC

        def layout(self):
            bazel_layout(self)

        def build(self):
            bazel = Bazel(self)
            bazel.build()
    """)
    BUILD = textwrap.dedent("""
    cc_library(
        name = "myapp",
        srcs = ["myapp.cpp"],
        hdrs = ["myapp.h"],
    )
    """)
    client = TestClient(path_with_spaces=False)
    bazel_root_dir = temp_folder(path_with_spaces=False).replace("\\", "/")
    client.save({
        "profile": profile,
        "profile_host": profile_host,
        "conanfile.py": conanfile,
        "WORKSPACE": "",
        ".bazelrc": f"startup --output_user_root={bazel_root_dir}",
        "main/BUILD": BUILD,
        "main/myapp.cpp": gen_function_cpp(name="myapp"),
        "main/myapp.h": gen_function_h(name="myapp"),

    })
    client.run("build . -pr:h profile_host -pr:b profile")
    libmyapp = os.path.join(client.current_folder, "bazel-bin", "main", "libmyapp.a")
    client.run_command(f'otool -hv {libmyapp}')
    assert "ARM64" in client.out



from conan.internal.api.detect.detect_api import default_cppstd
from conan.tools.build import cppstd_flag
from conan.internal.model.version import Version
from conan.test.utils.mocks import MockSettings, ConanFileMock


def _make_cppstd_flag(compiler, compiler_version, cppstd=None):
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({"compiler": compiler,
                                       "compiler.version": compiler_version,
                                       "compiler.cppstd": cppstd})
    return cppstd_flag(conanfile)


def _make_cppstd_default(compiler, compiler_version):
    return default_cppstd(compiler, Version(compiler_version))


class TestCompilerFlags:

    def test_gcc_cppstd_flags(self):
        assert _make_cppstd_flag("gcc", "4.2", "98") == "-std=c++98"
        assert _make_cppstd_flag("gcc", "4.2", "gnu98") == "-std=gnu++98"
        assert _make_cppstd_flag("gcc", "4.2", "11") is None
        assert _make_cppstd_flag("gcc", "4.2", "14") is None

        assert _make_cppstd_flag("gcc", "4.3", "98") == "-std=c++98"
        assert _make_cppstd_flag("gcc", "4.3", "gnu98") == "-std=gnu++98"
        assert _make_cppstd_flag("gcc", "4.3", "11") == "-std=c++0x"
        assert _make_cppstd_flag("gcc", "4.3", "14") is None

        assert _make_cppstd_flag("gcc", "4.6", "11") == '-std=c++0x'
        assert _make_cppstd_flag("gcc", "4.6", "14") is None

        assert _make_cppstd_flag("gcc", "4.7", "11") == '-std=c++11'
        assert _make_cppstd_flag("gcc", "4.7", "14") is None

        assert _make_cppstd_flag("gcc", "4.8", "11") == '-std=c++11'
        assert _make_cppstd_flag("gcc", "4.8", "14") == '-std=c++1y'
        assert _make_cppstd_flag("gcc", "4.8", "17") is None

        assert _make_cppstd_flag("gcc", "4.9", "11") == '-std=c++11'
        assert _make_cppstd_flag("gcc", "4.9", "14") == '-std=c++14'
        assert _make_cppstd_flag("gcc", "4.9", "17") is None

        assert _make_cppstd_flag("gcc", "5", "11") == '-std=c++11'
        assert _make_cppstd_flag("gcc", "5", "14") == '-std=c++14'
        assert _make_cppstd_flag("gcc", "5", "gnu14") == '-std=gnu++14'
        assert _make_cppstd_flag("gcc", "5", "17") == '-std=c++1z'
        assert _make_cppstd_flag("gcc", "5", "gnu17") == '-std=gnu++1z'

        assert _make_cppstd_flag("gcc", "5.1", "11") == '-std=c++11'
        assert _make_cppstd_flag("gcc", "5.1", "14") == '-std=c++14'
        assert _make_cppstd_flag("gcc", "5.1", "17") == '-std=c++1z'

        assert _make_cppstd_flag("gcc", "7", "11") == '-std=c++11'
        assert _make_cppstd_flag("gcc", "7", "14") == '-std=c++14'
        assert _make_cppstd_flag("gcc", "7", "17") == '-std=c++17'

        assert _make_cppstd_flag("gcc", "8", "11") == '-std=c++11'
        assert _make_cppstd_flag("gcc", "8", "14") == '-std=c++14'
        assert _make_cppstd_flag("gcc", "8", "17") == '-std=c++17'
        assert _make_cppstd_flag("gcc", "8", "20") == '-std=c++2a'
        assert _make_cppstd_flag("gcc", "8", "23") is None

        assert _make_cppstd_flag("gcc", "11", "11") == '-std=c++11'
        assert _make_cppstd_flag("gcc", "11", "14") == '-std=c++14'
        assert _make_cppstd_flag("gcc", "11", "17") == '-std=c++17'
        assert _make_cppstd_flag("gcc", "11", "20") == '-std=c++20'
        assert _make_cppstd_flag("gcc", "11", "23") == '-std=c++23'
        assert _make_cppstd_flag("gcc", "11", "26") is None

        assert _make_cppstd_flag("gcc", "14", "11") == '-std=c++11'
        assert _make_cppstd_flag("gcc", "14", "14") == '-std=c++14'
        assert _make_cppstd_flag("gcc", "14", "17") == '-std=c++17'
        assert _make_cppstd_flag("gcc", "14", "20") == '-std=c++20'
        assert _make_cppstd_flag("gcc", "14", "23") == '-std=c++23'
        assert _make_cppstd_flag("gcc", "14", "26") == '-std=c++26'

        assert _make_cppstd_flag("gcc", "15", "11") == '-std=c++11'
        assert _make_cppstd_flag("gcc", "15", "14") == '-std=c++14'
        assert _make_cppstd_flag("gcc", "15", "17") == '-std=c++17'
        assert _make_cppstd_flag("gcc", "15", "20") == '-std=c++20'
        assert _make_cppstd_flag("gcc", "15", "23") == '-std=c++23'
        assert _make_cppstd_flag("gcc", "15", "26") == '-std=c++26'

    def test_gcc_cppstd_defaults(self):
        assert _make_cppstd_default("gcc", "4") == "gnu98"
        assert _make_cppstd_default("gcc", "5") == "gnu98"
        assert _make_cppstd_default("gcc", "6") == "gnu14"
        assert _make_cppstd_default("gcc", "6.1") == "gnu14"
        assert _make_cppstd_default("gcc", "7.3") == "gnu14"
        assert _make_cppstd_default("gcc", "8.1") == "gnu14"
        assert _make_cppstd_default("gcc", "11") == "gnu17"
        assert _make_cppstd_default("gcc", "11.1") == "gnu17"
        assert _make_cppstd_default("gcc", "15.1") == "gnu17"

    def test_clang_cppstd_flags(self):
        assert _make_cppstd_flag("clang", "2.0", "98") is None
        assert _make_cppstd_flag("clang", "2.0", "gnu98") is None
        assert _make_cppstd_flag("clang", "2.0", "11") is None
        assert _make_cppstd_flag("clang", "2.0", "14") is None

        assert _make_cppstd_flag("clang", "2.1", "98") == "-std=c++98"
        assert _make_cppstd_flag("clang", "2.1", "gnu98") == "-std=gnu++98"
        assert _make_cppstd_flag("clang", "2.1", "11") == "-std=c++0x"
        assert _make_cppstd_flag("clang", "2.1", "14") is None

        assert _make_cppstd_flag("clang", "3.0", "11") == '-std=c++0x'
        assert _make_cppstd_flag("clang", "3.0", "14") is None

        assert _make_cppstd_flag("clang", "3.1", "11") == '-std=c++11'
        assert _make_cppstd_flag("clang", "3.1", "14") is None

        assert _make_cppstd_flag("clang", "3.4", "11") == '-std=c++11'
        assert _make_cppstd_flag("clang", "3.4", "14") == '-std=c++1y'
        assert _make_cppstd_flag("clang", "3.4", "17") is None

        assert _make_cppstd_flag("clang", "3.5", "11") == '-std=c++11'
        assert _make_cppstd_flag("clang", "3.5", "14") == '-std=c++14'
        assert _make_cppstd_flag("clang", "3.5", "17") == '-std=c++1z'

        assert _make_cppstd_flag("clang", "5", "11") == '-std=c++11'
        assert _make_cppstd_flag("clang", "5", "14") == '-std=c++14'
        assert _make_cppstd_flag("clang", "5", "gnu14") == '-std=gnu++14'
        assert _make_cppstd_flag("clang", "5", "17") == '-std=c++17'

        assert _make_cppstd_flag("clang", "5.1", "11") == '-std=c++11'
        assert _make_cppstd_flag("clang", "5.1", "14") == '-std=c++14'
        assert _make_cppstd_flag("clang", "5.1", "17") == '-std=c++17'

        for version in ["6", "7", "8", "9", "10", "11"]:
            assert _make_cppstd_flag("clang", version, "11") == '-std=c++11'
            assert _make_cppstd_flag("clang", version, "14") == '-std=c++14'
            assert _make_cppstd_flag("clang", version, "17") == '-std=c++17'
            assert _make_cppstd_flag("clang", version, "20") == '-std=c++2a'
            assert _make_cppstd_flag("clang", version, "23") is None

        assert _make_cppstd_flag("clang", "12", "11") == '-std=c++11'
        assert _make_cppstd_flag("clang", "12", "14") == '-std=c++14'
        assert _make_cppstd_flag("clang", "12", "17") == '-std=c++17'
        assert _make_cppstd_flag("clang", "12", "20") == '-std=c++20'
        assert _make_cppstd_flag("clang", "12", "23") == '-std=c++2b'
        assert _make_cppstd_flag("clang", "12", "26") is None

        assert _make_cppstd_flag("clang", "17", "11") == '-std=c++11'
        assert _make_cppstd_flag("clang", "17", "14") == '-std=c++14'
        assert _make_cppstd_flag("clang", "17", "17") == '-std=c++17'
        assert _make_cppstd_flag("clang", "17", "20") == '-std=c++20'
        assert _make_cppstd_flag("clang", "17", "23") == '-std=c++23'
        assert _make_cppstd_flag("clang", "17", "26") == '-std=c++26'

    def test_clang_cppstd_defaults(self):
        assert _make_cppstd_default("clang", "2") == "gnu98"
        assert _make_cppstd_default("clang", "2.1") == "gnu98"
        assert _make_cppstd_default("clang", "3.0") == "gnu98"
        assert _make_cppstd_default("clang", "3.1") == "gnu98"
        assert _make_cppstd_default("clang", "3.4") == "gnu98"
        assert _make_cppstd_default("clang", "3.5") == "gnu98"
        assert _make_cppstd_default("clang", "5") == "gnu98"
        assert _make_cppstd_default("clang", "5.1") == "gnu98"
        assert _make_cppstd_default("clang", "6") == "gnu14"
        assert _make_cppstd_default("clang", "7") == "gnu14"
        assert _make_cppstd_default("clang", "8") == "gnu14"
        assert _make_cppstd_default("clang", "9") == "gnu14"
        assert _make_cppstd_default("clang", "10") == "gnu14"
        assert _make_cppstd_default("clang", "11") == "gnu14"
        assert _make_cppstd_default("clang", "12") == "gnu14"
        assert _make_cppstd_default("clang", "13") == "gnu14"
        assert _make_cppstd_default("clang", "14") == "gnu14"
        assert _make_cppstd_default("clang", "15") == "gnu14"
        assert _make_cppstd_default("clang", "16") == "gnu17"

    def test_apple_clang_cppstd_flags(self):
        assert _make_cppstd_flag("apple-clang", "3.9", "98") is None
        assert _make_cppstd_flag("apple-clang", "3.9", "gnu98") is None
        assert _make_cppstd_flag("apple-clang", "3.9", "11") is None
        assert _make_cppstd_flag("apple-clang", "3.9", "14") is None

        assert _make_cppstd_flag("apple-clang", "4.0", "98") == "-std=c++98"
        assert _make_cppstd_flag("apple-clang", "4.0", "gnu98") == "-std=gnu++98"
        assert _make_cppstd_flag("apple-clang", "4.0", "11") == "-std=c++11"
        assert _make_cppstd_flag("apple-clang", "4.0", "14") is None

        assert _make_cppstd_flag("apple-clang", "5.0", "98") == "-std=c++98"
        assert _make_cppstd_flag("apple-clang", "5.0", "gnu98") == "-std=gnu++98"
        assert _make_cppstd_flag("apple-clang", "5.0", "11") == "-std=c++11"
        assert _make_cppstd_flag("apple-clang", "5.0", "14") is None

        assert _make_cppstd_flag("apple-clang", "5.1", "98") == "-std=c++98"
        assert _make_cppstd_flag("apple-clang", "5.1", "gnu98") == "-std=gnu++98"
        assert _make_cppstd_flag("apple-clang", "5.1", "11") == "-std=c++11"
        assert _make_cppstd_flag("apple-clang", "5.1", "14") == "-std=c++1y"

        assert _make_cppstd_flag("apple-clang", "6.1", "11") == '-std=c++11'
        assert _make_cppstd_flag("apple-clang", "6.1", "14") == '-std=c++14'
        assert _make_cppstd_flag("apple-clang", "6.1", "17") == "-std=c++1z"

        assert _make_cppstd_flag("apple-clang", "7", "11") == '-std=c++11'
        assert _make_cppstd_flag("apple-clang", "7", "14") == '-std=c++14'
        assert _make_cppstd_flag("apple-clang", "7", "17") == "-std=c++1z"

        assert _make_cppstd_flag("apple-clang", "8", "11") == '-std=c++11'
        assert _make_cppstd_flag("apple-clang", "8", "14") == '-std=c++14'
        assert _make_cppstd_flag("apple-clang", "8", "17") == "-std=c++1z"

        assert _make_cppstd_flag("apple-clang", "9", "11") == '-std=c++11'
        assert _make_cppstd_flag("apple-clang", "9", "14") == '-std=c++14'
        assert _make_cppstd_flag("apple-clang", "9", "17") == "-std=c++1z"

        assert _make_cppstd_flag("apple-clang", "9.1", "11") == '-std=c++11'
        assert _make_cppstd_flag("apple-clang", "9.1", "14") == '-std=c++14'
        assert _make_cppstd_flag("apple-clang", "9.1", "17") == "-std=c++17"
        assert _make_cppstd_flag("apple-clang", "9.1", "20") is None

        assert _make_cppstd_flag("apple-clang", "10.0", "17") == "-std=c++17"
        assert _make_cppstd_flag("apple-clang", "10.0", "20") == "-std=c++2a"
        assert _make_cppstd_flag("apple-clang", "11.0", "17") == "-std=c++17"
        assert _make_cppstd_flag("apple-clang", "11.0", "20") == "-std=c++2a"

        assert _make_cppstd_flag("apple-clang", "12.0", "17") == "-std=c++17"
        assert _make_cppstd_flag("apple-clang", "12.0", "20") == "-std=c++2a"
        assert _make_cppstd_flag("apple-clang", "12.0", "23") is None

        assert _make_cppstd_flag("apple-clang", "13.0", "17") == "-std=c++17"
        assert _make_cppstd_flag("apple-clang", "13.0", "gnu17") == "-std=gnu++17"
        assert _make_cppstd_flag("apple-clang", "13.0", "20") == "-std=c++20"
        assert _make_cppstd_flag("apple-clang", "13.0", "gnu20") == "-std=gnu++20"
        assert _make_cppstd_flag("apple-clang", "13.0", "23") == "-std=c++2b"
        assert _make_cppstd_flag("apple-clang", "13.0", "gnu23") == "-std=gnu++2b"

        assert _make_cppstd_flag("apple-clang", "14.0", "17") == "-std=c++17"
        assert _make_cppstd_flag("apple-clang", "14.0", "gnu17") == "-std=gnu++17"
        assert _make_cppstd_flag("apple-clang", "14.0", "20") == "-std=c++20"
        assert _make_cppstd_flag("apple-clang", "14.0", "gnu20") == "-std=gnu++20"
        assert _make_cppstd_flag("apple-clang", "14.0", "23") == "-std=c++2b"
        assert _make_cppstd_flag("apple-clang", "14.0", "gnu23") == "-std=gnu++2b"

        assert _make_cppstd_flag("apple-clang", "15.0", "17") == "-std=c++17"
        assert _make_cppstd_flag("apple-clang", "15.0", "gnu17") == "-std=gnu++17"
        assert _make_cppstd_flag("apple-clang", "15.0", "20") == "-std=c++20"
        assert _make_cppstd_flag("apple-clang", "15.0", "gnu20") == "-std=gnu++20"
        assert _make_cppstd_flag("apple-clang", "15.0", "23") == "-std=c++2b"
        assert _make_cppstd_flag("apple-clang", "15.0", "gnu23") == "-std=gnu++2b"
        assert _make_cppstd_flag("apple-clang", "15.0", "26") is None

        assert _make_cppstd_flag("apple-clang", "16.0", "17") == "-std=c++17"
        assert _make_cppstd_flag("apple-clang", "16.0", "gnu17") == "-std=gnu++17"
        assert _make_cppstd_flag("apple-clang", "16.0", "20") == "-std=c++20"
        assert _make_cppstd_flag("apple-clang", "16.0", "gnu20") == "-std=gnu++20"
        assert _make_cppstd_flag("apple-clang", "16.0", "23") == "-std=c++23"
        assert _make_cppstd_flag("apple-clang", "16.0", "gnu23") == "-std=gnu++23"
        assert _make_cppstd_flag("apple-clang", "16.0", "26") == "-std=c++26"
        assert _make_cppstd_flag("apple-clang", "16.0", "gnu26") == "-std=gnu++26"

        assert _make_cppstd_flag("apple-clang", "17.0", "17") == "-std=c++17"
        assert _make_cppstd_flag("apple-clang", "17.0", "gnu17") == "-std=gnu++17"
        assert _make_cppstd_flag("apple-clang", "17.0", "20") == "-std=c++20"
        assert _make_cppstd_flag("apple-clang", "17.0", "gnu20") == "-std=gnu++20"
        assert _make_cppstd_flag("apple-clang", "17.0", "23") == "-std=c++23"
        assert _make_cppstd_flag("apple-clang", "17.0", "gnu23") == "-std=gnu++23"
        assert _make_cppstd_flag("apple-clang", "17.0", "26") == "-std=c++26"
        assert _make_cppstd_flag("apple-clang", "17.0", "gnu26") == "-std=gnu++26"

    def test_apple_clang_cppstd_defaults(self):
        assert _make_cppstd_default("apple-clang", "2") == "gnu98"
        assert _make_cppstd_default("apple-clang", "3") == "gnu98"
        assert _make_cppstd_default("apple-clang", "4") == "gnu98"
        assert _make_cppstd_default("apple-clang", "5") == "gnu98"
        assert _make_cppstd_default("apple-clang", "6") == "gnu98"
        assert _make_cppstd_default("apple-clang", "7") == "gnu98"
        assert _make_cppstd_default("apple-clang", "8") == "gnu98"
        assert _make_cppstd_default("apple-clang", "9") == "gnu98"
        assert _make_cppstd_default("apple-clang", "10") == "gnu98"
        assert _make_cppstd_default("apple-clang", "11") == "gnu98"
        assert _make_cppstd_default("apple-clang", "12") == "gnu98"
        assert _make_cppstd_default("apple-clang", "13") == "gnu98"
        assert _make_cppstd_default("apple-clang", "14") == "gnu98"
        assert _make_cppstd_default("apple-clang", "15") == "gnu98"
        assert _make_cppstd_default("apple-clang", "16") == "gnu98"
        assert _make_cppstd_default("apple-clang", "17") == "gnu14"

    def test_visual_cppstd_flags(self):
        assert _make_cppstd_flag("msvc", "170", "11") is None
        assert _make_cppstd_flag("msvc", "170", "14") is None
        assert _make_cppstd_flag("msvc", "170", "17") is None

        assert _make_cppstd_flag("msvc", "180", "11") is None

        assert _make_cppstd_flag("msvc", "190", "14") == '/std:c++14'
        assert _make_cppstd_flag("msvc", "190", "17") == '/std:c++latest'

        assert _make_cppstd_flag("msvc", "191", "11") is None
        assert _make_cppstd_flag("msvc", "191", "14") == '/std:c++14'
        assert _make_cppstd_flag("msvc", "191", "17") == '/std:c++17'
        assert _make_cppstd_flag("msvc", "191", "20") == '/std:c++latest'

        assert _make_cppstd_flag("msvc", "192", "17") == '/std:c++17'
        assert _make_cppstd_flag("msvc", "192", "20") == '/std:c++20'

        assert _make_cppstd_flag("msvc", "193", "20") == '/std:c++20'
        assert _make_cppstd_flag("msvc", "193", "23") == '/std:c++latest'

    def test_visual_cppstd_defaults(self):
        assert _make_cppstd_default("msvc", "170") is None
        assert _make_cppstd_default("msvc", "180") is None
        assert _make_cppstd_default("msvc", "190") == "14"
        assert _make_cppstd_default("msvc", "191") == "14"
        assert _make_cppstd_default("msvc", "192") == "14"
        assert _make_cppstd_default("msvc", "193") == "14"

    def test_intel_cppstd_flag(self):
        assert _make_cppstd_flag("intel-cc", "2024.1", "gnu98") == '-std=gnu++98'
        assert _make_cppstd_flag("intel-cc", "2024.1", "11") == '-std=c++11'
        assert _make_cppstd_flag("intel-cc", "2024.1", "14") == '-std=c++14'
        assert _make_cppstd_flag("intel-cc", "2024.1", "17") == '-std=c++17'
        assert _make_cppstd_flag("intel-cc", "2024.1", "20") == '-std=c++20'

        assert _make_cppstd_flag("intel-cc", "2025.1", "17") == '-std=c++17'
        assert _make_cppstd_flag("intel-cc", "2025.1", "20") == '-std=c++20'
        assert _make_cppstd_flag("intel-cc", "2025.1", "23") == '-std=c++2b'

    def test_mcst_lcc_cppstd_defaults(self):
        assert _make_cppstd_default("mcst-lcc", "1.19") == "gnu98"
        assert _make_cppstd_default("mcst-lcc", "1.20") == "gnu98"
        assert _make_cppstd_default("mcst-lcc", "1.21") == "gnu98"
        assert _make_cppstd_default("mcst-lcc", "1.22") == "gnu98"
        assert _make_cppstd_default("mcst-lcc", "1.23") == "gnu98"
        assert _make_cppstd_default("mcst-lcc", "1.24") == "gnu14"
        assert _make_cppstd_default("mcst-lcc", "1.25") == "gnu14"

    def test_mcst_lcc_cppstd_flag(self):
        assert _make_cppstd_flag("mcst-lcc", "1.19", "98") == "-std=c++98"
        assert _make_cppstd_flag("mcst-lcc", "1.19", "11") is None
        assert _make_cppstd_flag("mcst-lcc", "1.19", "14") is None
        assert _make_cppstd_flag("mcst-lcc", "1.19", "17") is None
        assert _make_cppstd_flag("mcst-lcc", "1.19", "20") is None

        assert _make_cppstd_flag("mcst-lcc", "1.20", "98") == "-std=c++98"
        assert _make_cppstd_flag("mcst-lcc", "1.20", "11") is None
        assert _make_cppstd_flag("mcst-lcc", "1.20", "14") is None
        assert _make_cppstd_flag("mcst-lcc", "1.20", "17") is None
        assert _make_cppstd_flag("mcst-lcc", "1.20", "20") is None

        assert _make_cppstd_flag("mcst-lcc", "1.21", "98") == "-std=c++98"
        assert _make_cppstd_flag("mcst-lcc", "1.21", "11") == "-std=c++11"
        assert _make_cppstd_flag("mcst-lcc", "1.21", "14") == "-std=c++14"
        assert _make_cppstd_flag("mcst-lcc", "1.21", "17") is None
        assert _make_cppstd_flag("mcst-lcc", "1.21", "20") is None

        assert _make_cppstd_flag("mcst-lcc", "1.22", "98") == "-std=c++98"
        assert _make_cppstd_flag("mcst-lcc", "1.22", "11") == "-std=c++11"
        assert _make_cppstd_flag("mcst-lcc", "1.22", "14") == "-std=c++14"
        assert _make_cppstd_flag("mcst-lcc", "1.22", "17") is None
        assert _make_cppstd_flag("mcst-lcc", "1.22", "20") is None

        assert _make_cppstd_flag("mcst-lcc", "1.23", "98") == "-std=c++98"
        assert _make_cppstd_flag("mcst-lcc", "1.23", "11") == "-std=c++11"
        assert _make_cppstd_flag("mcst-lcc", "1.23", "14") == "-std=c++14"
        assert _make_cppstd_flag("mcst-lcc", "1.23", "17") is None
        assert _make_cppstd_flag("mcst-lcc", "1.23", "20") is None

        assert _make_cppstd_flag("mcst-lcc", "1.24", "98") == "-std=c++98"
        assert _make_cppstd_flag("mcst-lcc", "1.24", "11") == "-std=c++11"
        assert _make_cppstd_flag("mcst-lcc", "1.24", "14") == "-std=c++14"
        assert _make_cppstd_flag("mcst-lcc", "1.24", "17") == "-std=c++17"
        assert _make_cppstd_flag("mcst-lcc", "1.24", "20") is None

        assert _make_cppstd_flag("mcst-lcc", "1.25", "98") == "-std=c++98"
        assert _make_cppstd_flag("mcst-lcc", "1.25", "11") == "-std=c++11"
        assert _make_cppstd_flag("mcst-lcc", "1.25", "14") == "-std=c++14"
        assert _make_cppstd_flag("mcst-lcc", "1.25", "17") == "-std=c++17"
        assert _make_cppstd_flag("mcst-lcc", "1.25", "20") == "-std=c++2a"



from conan.api.conan_api import ConanAPI
from conan.test.utils.env import environment_update
from conan.api.model import PkgReference
from conan.api.model import RecipeReference
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_get_recipe_revisions():
    """
    Test the "api.list.recipe_revisions"
    """
    client = TestClient(default_server_user=True)
    for rev in range(1, 4):
        client.save({"conanfile.py": GenConanfile("foo", "1.0").with_build_msg(f"{rev}")})
        client.run("create .")
    client.run("upload * -r=default -c")

    api = ConanAPI(client.cache_folder)
    # Check the revisions locally
    ref = RecipeReference.loads("foo/1.0")
    sot = api.list.recipe_revisions(ref)
    sot = [r.repr_notime() for r in sot]
    assert ["foo/1.0#6707ddcdb444fd46f92d449d11700c5a",
            "foo/1.0#913d984a1b9b8a2821d8c4d4e9cf8d57",
            "foo/1.0#b87cdb893042ec4b371bc6aa82a0108f"] == sot


def test_get_package_revisions():
    """
    Test the "api.list.package_revisions"
    """
    client = TestClient(default_server_user=True)
    client.save({"conanfile.py": GenConanfile("foo", "1.0").with_package_file("f.txt",
                                                                              env_var="MYVAR")})
    for rev in range(3):
        with environment_update({"MYVAR": f"{rev}"}):
            client.run("create . ")
    client.run("upload * -r=default -c")

    api = ConanAPI(client.cache_folder)
    # Check the revisions locally
    pref = PkgReference.loads("foo/1.0#77ead28a5fd4216349b5b2181f4d32d4:"
                              "da39a3ee5e6b4b0d3255bfef95601890afd80709")
    sot = api.list.package_revisions(pref)
    sot = [r.repr_notime() for r in sot]
    assert ['foo/1.0#77ead28a5fd4216349b5b2181f4d32d4:'
            'da39a3ee5e6b4b0d3255bfef95601890afd80709#8b34fcf0543672de78ce1fe4f7fb3daa',
            'foo/1.0#77ead28a5fd4216349b5b2181f4d32d4:'
            'da39a3ee5e6b4b0d3255bfef95601890afd80709#5bb077c148d587da50ce9c3212370b5d',
            'foo/1.0#77ead28a5fd4216349b5b2181f4d32d4:'
            'da39a3ee5e6b4b0d3255bfef95601890afd80709#370f42f40d3f353a83a0f529ba2be1ce'] == sot


def test_search_recipes_no_user_channel_only():
    client = TestClient()
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name foo --version 1.0 --user user --channel channel")
    client.run("create . --name foo --version 1.0")
    client.run("list foo/1.0@")
    assert "foo/1.0@user/channel" not in client.out
    assert "foo/1.0" in client.out



import copy
import os
import textwrap

import pytest
from requests.models import Response

from conan.api.model import RecipeReference
from conan.internal.api.remotes.localdb import LocalDB
from conan.internal.util.files import save
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.env import environment_update
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient
from conan.test.utils.tools import TestRequester
from conan.test.utils.tools import TestServer


class TestAuthorize:

    @pytest.fixture(autouse=True)
    def setup(self):
        self.servers = {}
        self.ref = RecipeReference.loads("openssl/2.0.1@lasote/testing")
        # Create a default remote. R/W is not authorized for ref,
        # just for pepe, nacho and owner
        self.test_server = TestServer([(str(self.ref), "pepe,nacho@gmail.com")],  # read permissions
                                      [(str(self.ref), "pepe,nacho@gmail.com")],  # write permissions
                                      users={"lasote": "mypass",
                                             "pepe": "pepepass",
                                             "nacho@gmail.com": "nachopass"})  # exported creds
        self.servers["default"] = self.test_server

    def test_retries(self):
        """Bad login 2 times"""
        self.conan = TestClient(servers=self.servers, inputs=["bad", "1",
                                                              "bad2", "2",
                                                              "nacho@gmail.com", "nachopass"])
        self.conan.save({"conanfile.py": GenConanfile("openssl", "2.0.1")})
        self.conan.run("export . --user=lasote --channel=testing")
        errors = self.conan.run("upload %s -r default --only-recipe -v=quiet" % str(self.ref))
        # Check that return was  ok
        assert not errors
        # Check that upload was granted
        rev = self.test_server.server_store.get_last_revision(self.ref).revision
        ref = copy.copy(self.ref)
        ref.revision = rev
        assert os.path.exists(self.test_server.server_store.export(ref))
        assert "Please enter a password for user 'bad'" in self.conan.out
        assert "Please enter a password for user 'bad2'" in self.conan.out
        assert "Please enter a password for user 'nacho@gmail.com'" in self.conan.out

    def test_auth_with_env(self):

        def _upload_with_credentials(credentials, assert_error=False):
            cli = TestClient(servers=self.servers)
            cli.save({"conanfile.py": GenConanfile("openssl", "2.0.1")})
            cli.run("export . --user=lasote --channel=testing")
            with environment_update(credentials):
                cli.run("upload %s -r default --only-recipe" % str(self.ref),
                        assert_error=assert_error)
            return cli

        # Try with remote name in credentials
        client = _upload_with_credentials({"CONAN_PASSWORD_DEFAULT": "pepepass",
                                           "CONAN_LOGIN_USERNAME_DEFAULT": "pepe"})
        assert "Got username 'pepe' from environment" in client.out
        assert "Got password '******' from environment" in client.out

        # Try with generic password and login
        client = _upload_with_credentials({"CONAN_PASSWORD": "pepepass",
                                           "CONAN_LOGIN_USERNAME_DEFAULT": "pepe"})
        assert "Got username 'pepe' from environment" in client.out
        assert "Got password '******' from environment" in client.out

        # Try with generic password and generic login
        client = _upload_with_credentials({"CONAN_PASSWORD": "pepepass",
                                           "CONAN_LOGIN_USERNAME": "pepe"})
        assert "Got username 'pepe' from environment" in client.out
        assert "Got password '******' from environment" in client.out

        # Bad pass raise
        client = _upload_with_credentials({"CONAN_PASSWORD": "bad",
                                           "CONAN_LOGIN_USERNAME": "pepe"}, assert_error=True)
        assert "Authentication error in remote 'default'" in client.out

    def test_max_retries(self):
        """Bad login 3 times"""
        client = TestClient(servers=self.servers, inputs=["baduser", "badpass",
                                                          "baduser", "badpass2",
                                                          "baduser3", "badpass3"])
        client.save({"conanfile.py": GenConanfile("openssl", "2.0.1")})
        client.run("export . --user=lasote --channel=testing")
        errors = client.run("upload %s -r default --only-recipe" % str(self.ref), assert_error=True)
        # Check that return was not ok
        assert errors
        # Check that upload was not granted
        rev = self.servers["default"].server_store.get_last_revision(self.ref)
        assert rev is None

        # Check that login failed all times
        assert "Too many failed login attempts, bye!" in client.out

    def test_no_client_username_checks(self):
        """Checks whether client username checks are disabled."""

        # Try with a load of names that contain special characters
        client = TestClient(servers=self.servers, inputs=["some_random.special!characters",
                                                          "badpass",
                                                          "nacho@gmail.com", "nachopass"])

        client.save({"conanfile.py": GenConanfile("openssl", "2.0.1")})
        client.run("export . --user=lasote --channel=testing")
        client.run("upload %s -r default --only-recipe" % str(self.ref))

        # Check that upload was granted
        rev = self.test_server.server_store.get_last_revision(self.ref).revision
        ref = copy.copy(self.ref)
        ref.revision = rev
        assert os.path.exists(self.test_server.server_store.export(ref))
        assert ("Please enter a password for user 'some_random.special!characters' "
                "on remote 'default'") in client.out

    def test_authorize_disabled_remote(self):
        tc = TestClient(servers=self.servers)
        # Sanity check, this should not fail
        tc.run("remote login default pepe -p pepepass")
        tc.run("remote logout default")
        # This used to fail when the authentication was not possible for disabled remotes
        tc.run("remote disable default")
        tc.run("remote login default pepe -p pepepass")
        assert ("Changed user of remote 'default' "
                "from 'None' (anonymous) to 'pepe' (authenticated)") in tc.out


class TestAuthenticationTest:

    def test_unauthorized_during_capabilities(self):

        class RequesterMock(TestRequester):
            @staticmethod
            def get(url, **kwargs):
                resp_basic_auth = Response()
                resp_basic_auth.status_code = 200
                if "authenticate" in url:
                    assert kwargs["auth"].password == "PASSWORD!"
                    resp_basic_auth._content = b"TOKEN"
                    resp_basic_auth.headers = {"Content-Type": "text/plain"}
                elif "ping" in url:
                    resp_basic_auth.headers = {"Content-Type": "application/json",
                                               "X-Conan-Server-Capabilities": "revisions"}
                    bearer = getattr(kwargs["auth"], "bearer", None)
                    password = getattr(kwargs["auth"], "password", None)
                    assert bearer == "Bearer TOKEN" or password
                else:
                    assert "search" in url
                    assert kwargs["auth"].bearer == "Bearer TOKEN"
                    resp_basic_auth._content = b'{"results": []}'
                    resp_basic_auth.headers = {"Content-Type": "application/json"}
                return resp_basic_auth

        client = TestClient(requester_class=RequesterMock, default_server_user=True)
        client.run("remote login default user -p PASSWORD!")
        assert "Changed user of remote 'default' from 'None' (anonymous) to 'user'" in client.out
        client.run("search pkg -r=default")
        assert "ERROR: Recipe 'pkg' not found" in client.out


@pytest.mark.xfail(reason="This test is randomly failing")
def test_token_expired():
    server_folder = temp_folder()
    server_conf = textwrap.dedent("""
       [server]
       jwt_expire_minutes: 0.02
       authorize_timeout: 0
       disk_authorize_timeout: 0
       disk_storage_path: ./data
       updown_secret: 12345
       jwt_secret: mysecret
       port: 12345
       [read_permissions]
       */*@*/*: *
       [write_permissions]
       */*@*/*: admin
       """)
    save(os.path.join(server_folder, ".conan_server", "server.conf"), server_conf)
    server = TestServer(base_path=server_folder, users={"admin": "password", "other": "pass"})

    c = TestClient(servers={"default": server}, inputs=["admin", "password", "other", "pass"])
    c.save({"conanfile.py": GenConanfile()})
    c.run("create . --name=pkg --version=0.1 --user=user --channel=stable")
    c.run("upload * -r=default -c")
    localdb = LocalDB(c.cache_folder)
    user, token, _ = localdb.get_login(server.fake_url)
    assert user == "admin"
    assert token is not None

    import time
    time.sleep(3)
    c.users = {}
    c.run("remove * -c")
    c.run("install --requires=pkg/0.1@user/stable")
    assert "Remote 'default' needs authentication, obtaining credentials" in c.out
    user, token, _ = localdb.get_login(server.fake_url)
    assert user == "other"
    assert token is not None


def test_auth_username_space():
    server = TestServer(users={"super admin": "password"})
    c = TestClient(servers={"default": server}, inputs=["super admin", "password"])
    c.save({"conanfile.py": GenConanfile("pkg", "0.1")})
    c.run("export .")
    c.run("upload * -r=default -c")
    # it doesn't crash, it accepts user with space



import json
import textwrap

import pytest

from conan.api.model import RecipeReference
from conan.test.utils.tools import TestClient

conanfile = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.files import load, copy
    import os
    class Pkg(ConanFile):
        settings = "os"
        {requires}
        exports_sources = "myfile.txt"

        def generate(self):
            # Simulate an "imports"
            for dep in self.dependencies.values():
                dest_folder = os.path.join(self.build_folder, dep.ref.name)
                copy(self, "myfile.txt", dep.package_folder, dest_folder)

        def package(self):
            # Copy the ones from the dependencies
            copied = copy(self, "*myfile.txt", self.build_folder, self.package_folder, keep_path=True)
            # Copy the exported one
            copied = copy(self, "myfile.txt", self.source_folder, self.package_folder)
            assert len(copied) == 1

        def package_info(self):
            self.output.info("SELF OS: %s!!" % self.settings.os)
            self.output.info("SELF FILE: %s"
                % load(self, os.path.join(self.package_folder, "myfile.txt")))
            for d in os.listdir(self.package_folder):
                p = os.path.join(self.package_folder, d, "myfile.txt")
                if os.path.isfile(p):
                    self.output.info("DEP FILE %s: %s" % (d, load(self, p)))
        """)

pkgawin_01_id = "ebec3dc6d7f6b907b3ada0c3d3cdc83613a2b715"
pkganix_01_id = "9a4eb3c8701508aa9458b1a73d0633783ecc2270"
pkgb_01_id = "cee4c64978063c49773213b9c8f6b631c612a00b"
pkgb_012_id = "6b6343eeb8ffeb498b809bca9853fceb7dd0c078"
pkgc_01_id = "1bc0312cddbd3c076e666b84af9cc5ac3d263719"
pkgapp_01_id = "dda63a9bddbbe704d4858d67156d5dad0361dc19"


@pytest.fixture()
def client_setup():
    c = TestClient(light=True)
    c.save_home({"global.conf": "core.package_id:default_unknown_mode=recipe_revision_mode"})
    pkb_requirements = """
    def requirements(self):
        if self.settings.os == "Windows":
            self.requires("pkgawin/0.1")
        else:
            self.requires("pkganix/0.1")
    """
    files = {
        "pkga/conanfile.py": conanfile.format(requires=""),
        "pkga/myfile.txt": "HelloA",
        "pkgj/conanfile.py": conanfile.format(requires=""),
        "pkgj/myfile.txt": "HelloJ",
        "pkgb/conanfile.py": conanfile.format(requires=pkb_requirements),
        "pkgb/myfile.txt": "HelloB",
        "pkgc/conanfile.py": conanfile.format(requires='requires="pkgb/0.1"'),
        "pkgc/myfile.txt": "HelloC",
        "app1/conanfile.py": conanfile.format(requires='requires="pkgc/0.1"'),
        "app1/myfile.txt": "App1",
    }
    c.save(files)

    c.run("create pkga --name=pkgawin --version=0.1 -s os=Windows")
    c.run("create pkga --name=pkganix --version=0.1 -s os=Linux")
    c.run("create pkgb --name=pkgb --version=0.1 -s os=Windows")
    c.run("create pkgc --name=pkgc --version=0.1 -s os=Windows")
    c.run("create app1 --name=app1 --version=0.1 -s os=Windows")
    assert "app1/0.1: SELF FILE: App1" in c.out
    assert "app1/0.1: DEP FILE pkgawin: HelloA" in c.out
    assert "app1/0.1: DEP FILE pkgb: HelloB" in c.out
    assert "app1/0.1: DEP FILE pkgc: HelloC" in c.out
    return c


def test_single_config_centralized(client_setup):
    """ app1 -> pkgc/0.1 -> pkgb/0.1 -> pkgawin/0.1 or pkganix/0.1
    all versions are "0.1" exact (without pinning revision)
    lock app1.lock to lock graph including pkgawin/0.1#rev1 and pkganix/0.1#rev1
    changes in pkgawin/0.1#rev2 and pkganix/0.1#rev2 are excluded by lockfile
    a change in pkgb produces a new pkgb/0.1#rev2 that we want to test if works in app1 lockfile
    the app1 can be built in a single node, including all necessary dependencies
    the final lockfile will include pkgb/0.1#rev2, and not pkgb/0.1#rev1
    """
    c = client_setup
    # capture the initial lockfile of our product
    c.run("lock create --requires=app1/0.1@  --lockfile-out=app1.lock -s os=Windows")
    app1_lock = c.load("app1.lock")
    assert "pkgb/0.1#d03e920d532beeeb198cd886095bcca1" in app1_lock
    # Do an unrelated change in A, should not be used, this is not the change we are testing
    c.save({"pkga/myfile.txt": "ByeA World!!"})
    c.run("create pkga --name=pkgawin --version=0.1 -s os=Windows")

    # Do a change in B, this is the change that we want to test
    c.save({"pkgb/myfile.txt": "ByeB World!!"})

    # Test that pkgb/0.1 new revision works
    c.run("create pkgb --name=pkgb --version=0.1 -s os=Windows "
          "--lockfile=app1.lock --lockfile-out=app1_b_changed.lock")
    assert "pkgb/0.1: DEP FILE pkgawin: HelloA" in c.out

    # Now lets build the application, to see everything ok
    c.run("install --requires=app1/0.1@  --lockfile=app1_b_changed.lock "
          "--lockfile-out=app1_b_integrated.lock "
          "--build=missing  -s os=Windows")
    c.assert_listed_binary({"pkgawin/0.1": (pkgawin_01_id, "Cache"),
                            "pkgb/0.1": (pkgb_01_id, "Cache"),
                            "pkgc/0.1": (pkgc_01_id, "Build"),
                            "app1/0.1": (pkgapp_01_id, "Build")})
    assert "app1/0.1: DEP FILE pkgawin: HelloA" in c.out
    assert "app1/0.1: DEP FILE pkgb: ByeB World!!" in c.out

    # All good! We can get rid of the now unused pkgb/0.1 version in the lockfile
    c.run("lock create --requires=app1/0.1@ --lockfile=app1_b_integrated.lock "
          "--lockfile-out=app1_clean.lock -s os=Windows --lockfile-clean")
    app1_clean = c.load("app1_clean.lock")
    assert "pkgb/0.1#d03e920d532beeeb198cd886095bcca1" not in app1_clean
    assert "pkgb/0.1#bb12977c3353d7633b34d55a926fe58c" in app1_clean


def test_single_config_centralized_out_range(client_setup):
    """ same scenario as "test_single_config_centralized()"
    but pkgc pin the exact revision of pkgb/0.1#rev1
    But pkgb/0.1 change produces pkgb/0.1#rev2, which doesn't match the pinned revisions rev1
    Nothing to build in the app1, and the final lockfile doesn't change at all
    """
    # Out of range in revisions means a pinned revision, new revision will not match
    c = client_setup
    c.save({"pkgc/conanfile.py":
            conanfile.format(requires='requires="pkgb/0.1#d03e920d532beeeb198cd886095bcca1"')})
    c.run("create pkgc --name=pkgc --version=0.1 -s os=Windows")
    c.run("create app1 --name=app1 --version=0.1 -s os=Windows --build=missing")
    c.run("lock create --requires=app1/0.1@ --lockfile-out=app1.lock -s os=Windows")
    app1_lock = c.load("app1.lock")
    assert "pkgb/0.1#d03e920d532beeeb198cd886095bcca1" in app1_lock

    # Do an unrelated change in A, should not be used, this is not the change we are testing
    c.save({"pkga/myfile.txt": "ByeA World!!"})
    c.run("create pkga --name=pkgawin --version=0.1 -s os=Windows")

    # Do a change in B, this is the change that we want to test
    c.save({"pkgb/myfile.txt": "ByeB World!!"})

    # Test that pkgb/1.0 works (but it is not matching the pinned revision)
    c.run("create pkgb --name=pkgb --version=0.1 -s os=Windows "
          "--lockfile=app1.lock --lockfile-out=app1_b_changed.lock")
    assert "pkgb/0.1: DEP FILE pkgawin: HelloA" in c.out
    app1_b_changed = c.load("app1_b_changed.lock")
    assert "pkgb/0.1#bb12977c3353d7633b34d55a926fe58c" in app1_b_changed
    assert "pkgb/0.1#d03e920d532beeeb198cd886095bcca1" in app1_b_changed

    # Now lets build the application, to see everything ok
    c.run("install --requires=app1/0.1@  --lockfile=app1_b_changed.lock "
          "--lockfile-out=app1_b_integrated.lock "
          "--build=missing  -s os=Windows")
    # Nothing changed, the change is outside the range, app1 not affected!!
    c.assert_listed_binary({"pkgawin/0.1": (pkgawin_01_id, "Cache"),
                            "pkgb/0.1": (pkgb_01_id, "Cache"),
                            "pkgc/0.1": ("7dc09fd93c15a010373b013c3a44fc94fc9d3226", "Cache"),
                            "app1/0.1": ("96324a4bf6d4bba4f697919a435eca6d746c2d18", "Cache")})
    assert "app1/0.1: DEP FILE pkgawin: HelloA" in c.out
    assert "app1/0.1: DEP FILE pkgb: HelloB" in c.out

    # All good! We can get rid of the now unused pkgb/1.0 version in the lockfile
    c.run("lock create --requires=app1/0.1@ --lockfile=app1_b_integrated.lock "
          "--lockfile-out=app1_clean.lock -s os=Windows --lockfile-clean")
    app1_clean = c.load("app1_clean.lock")
    assert "pkgb/0.1#d03e920d532beeeb198cd886095bcca1" in app1_clean
    assert "pkgb/0.1#504bc7152c72b49c99a6f16733fb2ff6" not in app1_clean


def test_single_config_centralized_change_dep(client_setup):
    """ same scenario as "test_single_config_centralized()".
    But pkgb/0.1 change producing pkgb/0.1#rev2, and changes dependency from pkgA=>pkgJ
    """
    c = client_setup
    c.run("lock create --requires=app1/0.1@ --lockfile-out=app1.lock -s os=Windows")

    # Do an unrelated change in A, should not be used, this is not the change we are testing
    c.save({"pkga/myfile.txt": "ByeA World!!"})
    c.run("create pkga --name=pkgawin --version=0.1 -s os=Windows")

    # Build new package alternative J
    c.run("create pkgj --name=pkgj --version=0.1 -s os=Windows")

    # Do a change in B, this is the change that we want to test, remove pkgA, replace with pkgJ
    c.save({"pkgb/conanfile.py": conanfile.format(requires='requires="pkgj/0.1"'),
            "pkgb/myfile.txt": "ByeB World!!"})

    # Test that pkgb/0.1 new revision works
    c.run("create pkgb --name=pkgb --version=0.1 -s os=Windows --lockfile-partial "
          "--lockfile=app1.lock --lockfile-out=app1_b_changed.lock")
    assert "pkgb/0.1: DEP FILE pkgj: HelloJ" in c.out
    # Build new package alternative J, it won't be included, already locked in this create
    c.run("create pkgj --name=pkgj --version=0.1 -s os=Windows")

    # Now lets build the application, to see everything ok
    c.run("install --requires=app1/0.1@  --lockfile=app1_b_changed.lock "
          "--lockfile-out=app1_b_integrated.lock "
          "--build=missing  -s os=Windows")
    assert "pkga/" not in c.out
    c.assert_listed_binary({"pkgj/0.1": (pkgawin_01_id, "Cache"),
                            "pkgb/0.1": ("6142fb85ccd4e94afad85a8d01a87234eefa5600", "Cache"),
                            "pkgc/0.1": ("93cfcbc8109eedf4211558258ff5a844fdb62cca", "Build"),
                            "app1/0.1": ("eb241e40d370e1e1b0fd516aff6ffff72de1e37d", "Build")})
    assert "app1/0.1: DEP FILE pkgj: HelloJ" in c.out
    assert "app1/0.1: DEP FILE pkgb: ByeB World!!" in c.out

    # All good! We can get rid of the now unused pkgb/1.0 version in the lockfile
    c.run("lock create --requires=app1/0.1@ --lockfile=app1_b_integrated.lock "
          "--lockfile-out=app1_clean.lock -s os=Windows --lockfile-clean")
    app1_clean = c.load("app1_clean.lock")
    assert "pkgj/0.1" in app1_clean
    assert "pkgb/0.1" in app1_clean
    assert "pkga" not in app1_clean


def test_multi_config_centralized(client_setup):
    """ same scenario as above, but now we want to manage 2 configurations Windows & Linux
    When pkgB is changed, it is built for both, and produces app1_win.lock and app2_linux.lock
    With those, app1 can be built in a single node for both configurations. After building
    app1, the 2 lockfiles can be cleaned (removing the old pkgb/0.1#rev1, leaving pkgb/0.1#rev2
    in the lock)
    The 2 final lockfiles can be "merged" in a single one for next iteration
    """
    c = client_setup
    # capture the initial lockfile of our product
    c.run("lock create --requires=app1/0.1@ --lockfile-out=app1.lock -s os=Windows")
    c.run("lock create --requires=app1/0.1@ --lockfile=app1.lock --lockfile-out=app1.lock "
          "-s os=Linux")
    app1_lock = c.load("app1.lock")
    assert "pkgb/0.1#d03e920d532beeeb198cd886095bcca1" in app1_lock
    assert "pkgawin/0.1#2f297d19d9ee4827caf97071de449a54" in app1_lock
    assert "pkganix/0.1#2f297d19d9ee4827caf97071de449a54" in app1_lock

    # Do an unrelated change in A, should not be used, this is not the change we are testing
    c.save({"pkga/myfile.txt": "ByeA World!!"})
    c.run("create pkga --name=pkgawin --version=0.1 -s os=Windows")
    c.run("create pkga --name=pkganix --version=0.1 -s os=Linux")

    # Do a change in B, this is the change that we want to test
    c.save({"pkgb/myfile.txt": "ByeB World!!"})

    # Test that pkgb/0.1 works
    c.run("create pkgb --name=pkgb --version=0.1 -s os=Windows "
          "--lockfile=app1.lock --lockfile-out=app1_win.lock")
    assert "pkgb/0.1: DEP FILE pkgawin: HelloA" in c.out
    c.run("create pkgb --name=pkgb --version=0.1 -s os=Linux "
          "--lockfile=app1.lock --lockfile-out=app1_nix.lock")
    assert "pkgb/0.1: DEP FILE pkganix: HelloA" in c.out

    # Now lets build the application in Windows, to see everything ok
    c.run("install --requires=app1/0.1@  --lockfile=app1_win.lock --lockfile-out=app1_win.lock "
          "--build=missing  -s os=Windows")
    c.assert_listed_binary({"pkgawin/0.1": (pkgawin_01_id, "Cache"),
                            "pkgb/0.1": (pkgb_01_id, "Cache"),
                            "pkgc/0.1": (pkgc_01_id, "Build"),
                            "app1/0.1": (pkgapp_01_id, "Build")})
    assert "app1/0.1: DEP FILE pkgawin: HelloA" in c.out
    assert "app1/0.1: DEP FILE pkgb: ByeB World!!" in c.out

    # Now lets build the application in Linux, to see everything ok
    c.run("install --requires=app1/0.1@  --lockfile=app1_nix.lock --lockfile-out=app1_nix.lock "
          "--build=missing  -s os=Linux")
    c.assert_listed_binary({"pkganix/0.1": (pkganix_01_id, "Cache"),
                            "pkgb/0.1": (pkgb_012_id, "Cache"),
                            "pkgc/0.1": ("62b3834a578b45bb303925e1e9cfe0dd9908486e", "Build"),
                            "app1/0.1": ("6a589308a14c21c9082620be4a63240017665e38", "Build")})

    assert "app1/0.1: DEP FILE pkganix: HelloA" in c.out
    assert "app1/0.1: DEP FILE pkgb: ByeB World!!" in c.out

    # All good! We can get rid of the now unused pkgb/0.1 version in the lockfile
    c.run("lock create --requires=app1/0.1@ --lockfile=app1_win.lock "
          "--lockfile-out=app1_win.lock -s os=Windows --lockfile-clean")
    c.run("lock create --requires=app1/0.1@ --lockfile=app1_nix.lock "
          "--lockfile-out=app1_nix.lock -s os=Linux --lockfile-clean")

    # Finally, merge the 2 clean lockfiles, for keeping just 1 for next iteration
    c.run("lock merge --lockfile=app1_win.lock --lockfile=app1_nix.lock "
          "--lockfile-out=app1_final.lock")
    app1_clean = c.load("app1_final.lock")
    assert "pkgb/0.1#d03e920d532beeeb198cd886095bcca1" not in app1_clean
    assert "pkgawin/0.1#2f297d19d9ee4827caf97071de449a54" in app1_clean
    assert "pkganix/0.1#2f297d19d9ee4827caf97071de449a54" in app1_clean


def test_single_config_decentralized(client_setup):
    """ same scenario as "test_single_config_centralized()", but distributing the build in
    different build servers, using the "build-order"
    """
    c = client_setup
    # capture the initial lockfile of our product
    c.run("lock create --requires=app1/0.1@  --lockfile-out=app1.lock -s os=Windows")
    app1_lock = c.load("app1.lock")
    assert "pkgb/0.1#d03e920d532beeeb198cd886095bcca1" in app1_lock

    # Do an unrelated change in A, should not be used, this is not the change we are testing
    c.save({"pkga/myfile.txt": "ByeA World!!"})
    c.run("create pkga --name=pkgawin --version=0.1 -s os=Windows")

    # Do a change in B, this is the change that we want to test
    c.save({"pkgb/myfile.txt": "ByeB World!!"})

    # Test that pkgb/0.1 works
    c.run("create pkgb --name=pkgb --version=0.1 -s os=Windows "
          "--lockfile=app1.lock --lockfile-out=app1_b_changed.lock")
    assert "pkgb/0.1: DEP FILE pkgawin: HelloA" in c.out

    # Now lets build the application, to see everything ok
    c.run("graph build-order --requires=app1/0.1@ --lockfile=app1_b_changed.lock "
          "--build=missing --format=json -s os=Windows", redirect_stdout="build_order.json")
    json_file = c.load("build_order.json")
    to_build = json.loads(json_file)
    level0 = to_build[0]
    assert len(level0) == 1
    pkgawin = level0[0]
    assert pkgawin["ref"] == "pkgawin/0.1#2f297d19d9ee4827caf97071de449a54"
    assert pkgawin["packages"][0][0]["binary"] == "Cache"
    level1 = to_build[1]
    assert len(level1) == 1
    pkgb = level1[0]
    assert pkgb["ref"] == "pkgb/0.1#bb12977c3353d7633b34d55a926fe58c"
    assert pkgb["packages"][0][0]["binary"] == "Cache"

    for level in to_build:
        for elem in level:
            ref = RecipeReference.loads(elem["ref"])
            for package in elem["packages"][0]:  # Assumes no dependencies between packages
                binary = package["binary"]
                package_id = package["package_id"]
                if binary != "Build":
                    continue
                # TODO: The options are completely missing
                c.run(
                    "install --requires=%s@ --build=%s@ --lockfile=app1_b_changed.lock  -s os=Windows"
                    % (ref, ref))
                c.assert_listed_binary(
                    {str(ref): (package_id, "Build"),
                     "pkgawin/0.1": (pkgawin_01_id, "Cache"),
                     "pkgb/0.1": (pkgb_01_id, "Cache")})
                assert "DEP FILE pkgawin: HelloA" in c.out
                assert "DEP FILE pkgb: ByeB World!!" in c.out

    # Just to make sure that the for-loops have been executed
    assert "app1/0.1: DEP FILE pkgb: ByeB World!!" in c.out


def test_multi_config_decentralized(client_setup):
    """ same scenario as "test_multi_config_centralized()", but distributing the build in
    different build servers, using the "build-order"
    """
    c = client_setup
    # capture the initial lockfile of our product
    c.run("lock create --requires=app1/0.1@ --lockfile-out=app1.lock -s os=Windows")
    c.run("lock create --requires=app1/0.1@ --lockfile=app1.lock --lockfile-out=app1.lock "
          "-s os=Linux")

    # Do an unrelated change in A, should not be used, this is not the change we are testing
    c.save({"pkga/myfile.txt": "ByeA World!!"})
    c.run("create pkga --name=pkgawin --version=0.1 -s os=Windows")
    c.run("create pkga --name=pkganix --version=0.1 -s os=Linux")

    # Do a change in B, this is the change that we want to test
    c.save({"pkgb/myfile.txt": "ByeB World!!"})

    # Test that pkgb/0.1 new revision works
    c.run("create pkgb --name=pkgb --version=0.1 -s os=Windows "
          "--lockfile=app1.lock --lockfile-out=app1_win.lock")
    assert "pkgb/0.1: DEP FILE pkgawin: HelloA" in c.out
    c.run("create pkgb --name=pkgb --version=0.1 -s os=Linux "
          "--lockfile=app1.lock --lockfile-out=app1_nix.lock")
    assert "pkgb/0.1: DEP FILE pkganix: HelloA" in c.out

    # Now lets build the application, to see everything ok, for all the configs
    c.run("graph build-order --requires=app1/0.1@ --lockfile=app1_win.lock "
          "--build=missing --format=json -s os=Windows", redirect_stdout="app1_win.json")
    c.run("graph build-order --requires=app1/0.1@ --lockfile=app1_nix.lock "
          "--build=missing --format=json -s os=Linux", redirect_stdout="app1_nix.json")
    c.run("graph build-order-merge --file=app1_win.json --file=app1_nix.json"
          " --format=json", redirect_stdout="build_order.json")

    json_file = c.load("build_order.json")
    to_build = json.loads(json_file)
    level0 = to_build[0]
    assert len(level0) == 2
    pkgawin = level0[0]
    assert pkgawin["ref"] == "pkgawin/0.1#2f297d19d9ee4827caf97071de449a54"
    assert pkgawin["packages"][0][0]["binary"] == "Cache"
    pkgawin = level0[1]
    assert pkgawin["ref"] == "pkganix/0.1#2f297d19d9ee4827caf97071de449a54"
    assert pkgawin["packages"][0][0]["binary"] == "Cache"
    level1 = to_build[1]
    assert len(level1) == 1
    pkgb = level1[0]
    assert pkgb["ref"] == "pkgb/0.1#bb12977c3353d7633b34d55a926fe58c"
    assert pkgb["packages"][0][0]["binary"] == "Cache"

    for level in to_build:
        for elem in level:
            ref = elem["ref"]
            ref_without_rev = ref.split("#")[0]
            if "@" not in ref:
                ref = ref.replace("#", "@#")
            for package in elem["packages"][0]:  # Assumes no dependencies between packages
                binary = package["binary"]
                package_id = package["package_id"]
                if binary != "Build":
                    continue
                # TODO: The options are completely missing
                filenames = package["filenames"]
                lockfile = filenames[0] + ".lock"
                the_os = "Windows" if "win" in lockfile else "Linux"
                c.run("install --requires=%s --build=%s --lockfile=%s -s os=%s"
                      % (ref, ref, lockfile, the_os))
                c.assert_listed_binary({str(ref_without_rev): (package_id, "Build")})

                if the_os == "Windows":
                    c.assert_listed_binary(
                        {"pkgawin/0.1": (pkgawin_01_id, "Cache"),
                         "pkgb/0.1": (pkgb_01_id, "Cache")})
                    assert "DEP FILE pkgawin: HelloA" in c.out
                    assert "DEP FILE pkgb: ByeB World!!" in c.out
                else:
                    c.assert_listed_binary(
                        {"pkganix/0.1": (pkganix_01_id, "Cache"),
                         "pkgb/0.1": (pkgb_012_id, "Cache")})
                    assert "DEP FILE pkganix: HelloA" in c.out
                    assert "DEP FILE pkgb: ByeB World!!" in c.out

    # Just to make sure that the for-loops have been executed
    assert "app1/0.1: DEP FILE pkgb: ByeB World!!" in c.out



import os
import textwrap

from conan.test.utils.tools import TestClient


def test_vs_layout_subproject():
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.microsoft import vs_layout
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "MSBuildToolchain"
            def layout(self):
                self.folders.root = ".."
                self.folders.subproject = "pkg"
                vs_layout(self)
        """)
    c.save({"pkg/conanfile.py": conanfile})
    c.run("install pkg")
    assert os.path.isfile(os.path.join(c.current_folder, "pkg", "conan", "conantoolchain.props"))


def test_vs_layout_error():
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.microsoft import vs_layout
        class Pkg(ConanFile):
            settings = "os", "compiler", "arch"
            def layout(self):
                vs_layout(self)
        """)
    c.save({"conanfile.py": conanfile})
    c.run("install .", assert_error=True)
    assert "The 'vs_layout' requires the 'build_type' setting" in c.out
    conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.microsoft import vs_layout
            class Pkg(ConanFile):
                settings = "os", "compiler", "build_type"
                def layout(self):
                    vs_layout(self)
            """)
    c.save({"conanfile.py": conanfile})
    c.run("install .", assert_error=True)
    assert "The 'vs_layout' requires the 'arch' setting" in c.out
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.microsoft import vs_layout
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            def layout(self):
                vs_layout(self)
        """)
    c.save({"conanfile.py": conanfile})
    c.run("install . -s arch=riscv64", assert_error=True)
    assert "The 'vs_layout' doesn't work with the arch 'riscv64'" in c.out
    assert "Accepted architectures: 'x86', 'x86_64', 'armv7', 'armv8'" in c.out



import json
import os
import re
import textwrap
from collections import OrderedDict

import pytest

from conan.test.utils.tools import NO_SETTINGS_PACKAGE_ID
from conan.test.utils.tools import TestClient, TestServer, GenConanfile
from conan.internal.util.files import mkdir, save


@pytest.fixture()
def client():
    c = TestClient(default_server_user=True)
    c.save_home({"settings.yml": "os: [Windows, Macos, Linux, FreeBSD]\nos_build: [Windows, Macos]",
                 "profiles/default": "[settings]\nos=Windows"})
    return c


def test_install_reference_txt(client):
    # Test to check the "conan install <path> <reference>" command argument
    client.save({"conanfile.txt": ""})
    client.run("install .")
    assert "conanfile.txt" in client.out


def test_install_reference_error(client):
    # Test to check the "conan install <path> <reference>" command argument
    client.run("install --requires=pkg/0.1@myuser/testing --user=user --channel=testing", assert_error=True)
    assert "ERROR: Can't use --name, --version, --user or --channel arguments with --requires" in client.out
    client.save({"conanfile.py": GenConanfile("pkg", "1.0")})
    client.run("install . --channel=testing", assert_error=True)
    assert "Can't specify channel 'testing' without user" in client.out


def test_install_args_error():
    c = TestClient()
    c.run("install . --requires=zlib/1.0", assert_error=True)
    assert "--requires and --tool-requires arguments are incompatible" in c.out


def test_four_subfolder_install(client):
    # https://github.com/conan-io/conan/issues/3950
    client.save({"path/to/sub/folder/conanfile.txt": ""})
    # If this doesn't, fail, all good
    client.run(" install path/to/sub/folder")


@pytest.mark.artifactory_ready
def test_install_system_requirements(client):
    client.save({"conanfile.py": textwrap.dedent("""
        from conan import ConanFile
        class MyPkg(ConanFile):
            def system_requirements(self):
                self.output.info("Running system requirements!!")
        """)})
    client.run(" install .")
    assert "Running system requirements!!" in client.out
    client.run("export . --name=pkg --version=0.1 --user=lasote --channel=testing")
    client.run(" install --requires=pkg/0.1@lasote/testing --build='*'")
    assert "Running system requirements!!" in client.out
    client.run("upload * --confirm -r default")
    client.run('remove "*" -c')
    client.run(" install --requires=pkg/0.1@lasote/testing")
    assert "Running system requirements!!" in client.out


def test_install_transitive_pattern(client):
    # Make sure a simple conan install doesn't fire package_info() so self.package_folder breaks
    client.save({"conanfile.py": textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            options = {"shared": [True, False, "header"]}
            default_options = {"shared": False}
            def package_info(self):
                self.output.info("PKG OPTION: %s" % self.options.shared)
        """)})
    client.run("create . --name=pkg --version=0.1 --user=user --channel=testing -o shared=True")
    assert "pkg/0.1@user/testing: PKG OPTION: True" in client.out
    client.save({"conanfile.py": textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            requires = "pkg/0.1@user/testing"
            options = {"shared": [True, False, "header"]}
            default_options = {"shared": False}
            def package_info(self):
                self.output.info("PKG2 OPTION: %s" % self.options.shared)
        """)})

    client.run("create . --name=pkg2 --version=0.1 --user=user --channel=testing -o *:shared=True")
    assert "pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "pkg2/0.1@user/testing: PKG2 OPTION: True" in client.out
    client.run(" install --requires=pkg2/0.1@user/testing -o *:shared=True")
    assert "pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "pkg2/0.1@user/testing: PKG2 OPTION: True" in client.out
    # Priority of non-scoped options
    client.run("create . --name=pkg2 --version=0.1 --user=user --channel=testing -o shared=header -o *:shared=True")
    assert "pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    client.run(" install --requires=pkg2/0.1@user/testing -o shared=header -o *:shared=True")
    assert "pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    # Prevalence of exact named option
    client.run("create . --name=pkg2 --version=0.1 --user=user --channel=testing -o *:shared=True -o pkg2*:shared=header")
    assert "pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    client.run(" install --requires=pkg2/0.1@user/testing -o *:shared=True -o pkg2*:shared=header")
    assert "pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    # Prevalence of exact named option reverse
    client.run("create . --name=pkg2 --version=0.1 --user=user --channel=testing -o *:shared=True -o pkg/*:shared=header "
               "--build=missing")
    assert "pkg/0.1@user/testing: PKG OPTION: header" in client.out
    assert "pkg2/0.1@user/testing: PKG2 OPTION: True" in client.out
    client.run(" install --requires=pkg2/0.1@user/testing -o *:shared=True -o pkg/*:shared=header")
    assert "pkg/0.1@user/testing: PKG OPTION: header" in client.out
    assert "pkg2/0.1@user/testing: PKG2 OPTION: True" in client.out
    # Prevalence of alphabetical pattern
    client.run("create . --name=pkg2 --version=0.1 --user=user --channel=testing -o *:shared=True -o pkg2*:shared=header")
    assert "pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    client.run(" install --requires=pkg2/0.1@user/testing -o *:shared=True -o pkg2*:shared=header")
    assert "pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    # Prevalence of last match, even first pattern match
    client.run("create . --name=pkg2 --version=0.1 --user=user --channel=testing -o pkg2*:shared=header -o *:shared=True")
    assert "pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "pkg2/0.1@user/testing: PKG2 OPTION: True" in client.out
    client.run(" install --requires=pkg2/0.1@user/testing -o pkg2*:shared=header -o *:shared=True")
    assert "pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "pkg2/0.1@user/testing: PKG2 OPTION: True" in client.out
    # Prevalence and override of alphabetical pattern
    client.run("create . --name=pkg2 --version=0.1 --user=user --channel=testing -o *:shared=True -o pkg*:shared=header")
    assert "pkg/0.1@user/testing: PKG OPTION: header" in client.out
    assert "pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    client.run(" install --requires=pkg2/0.1@user/testing -o *:shared=True -o pkg*:shared=header")
    assert "pkg/0.1@user/testing: PKG OPTION: header" in client.out
    assert "pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out


def test_install_package_folder(client):
    # Make sure a simple conan install doesn't fire package_info() so self.package_folder breaks
    client.save({"conanfile.py": textwrap.dedent("""\
        from conan import ConanFile
        import os
        class Pkg(ConanFile):
            def package_info(self):
                self.dummy_doesnt_exist_not_break
                self.output.info("Hello")
                self.env_info.PATH = os.path.join(self.package_folder, "bin")
        """)})
    client.run("install .")
    assert "Hello" not in client.out


def test_install_cwd(client):
    client.save({"conanfile.py": GenConanfile("hello", "0.1").with_setting("os")})
    client.run("export . --user=lasote --channel=stable")
    client.save({"conanfile.txt": "[requires]\nhello/0.1@lasote/stable"}, clean_first=True)

    client.run("install . --build=missing -s os_build=Windows")
    assert "hello/0.1@lasote/stable#a20db3358243e96aa07f654eaada1564 - Cache" in client.out


def test_install_with_profile(client):
    # Test for https://github.com/conan-io/conan/pull/2043
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class TestConan(ConanFile):
            settings = "os"
            def requirements(self):
                self.output.info("PKGOS=%s" % self.settings.os)
        """)

    client.save({"conanfile.py": conanfile})
    save(os.path.join(client.paths.profiles_path, "myprofile"), "[settings]\nos=Linux")
    client.run("install . -pr=myprofile --build='*'")
    assert "PKGOS=Linux" in client.out
    mkdir(os.path.join(client.current_folder, "myprofile"))
    client.run("install . -pr=myprofile")
    save(os.path.join(client.paths.profiles_path, "myotherprofile"), "[settings]\nos=FreeBSD")
    client.run("install . -pr=myotherprofile")
    assert "PKGOS=FreeBSD" in client.out
    client.save({"myotherprofile": "Some garbage without sense [garbage]"})
    client.run("install . -pr=myotherprofile")
    assert "PKGOS=FreeBSD" in client.out
    client.run("install . -pr=./myotherprofile", assert_error=True)
    assert "Error while parsing line 0" in client.out


def test_install_with_path_errors(client):
    # Install without path param allowed, but nothing found
    client.run("install", assert_error=True)
    assert "Conanfile not found" in client.out

    # Install without path param allowed, but nothing found
    client.run("install . --requires=foo/1.0", assert_error=True)
    assert "--requires and --tool-requires arguments are incompatible with [path]" in client.out

    # Path with wrong conanfile.txt path
    client.run("install not_real_dir/conanfile.txt", assert_error=True)
    assert "Conanfile not found" in client.out


def test_install_argument_order(client):
    # https://github.com/conan-io/conan/issues/2520
    conanfile_boost = textwrap.dedent("""
        from conan import ConanFile
        class BoostConan(ConanFile):
            name = "boost"
            version = "0.1"
            options = {"shared": [True, False]}
            default_options = {"shared": True}
        """)
    conanfile = GenConanfile().with_require("boost/0.1")

    client.save({"conanfile.py": conanfile,
                 "conanfile_boost.py": conanfile_boost})
    client.run("create conanfile_boost.py ")
    client.run("install . -o boost/*:shared=True --build=missing")
    output_0 = client.out
    client.run("install . -o boost/*:shared=True --build missing")
    output_1 = client.out
    client.run("install -o boost/*:shared=True . --build missing")
    output_2 = client.out
    client.run("install -o boost/*:shared=True --build missing .")
    output_3 = client.out
    assert "ERROR" not in output_3
    assert output_0 == output_1
    assert output_1 == output_2
    assert output_2 == output_3

    client.run("install -o boost/*:shared=True --build boost . --build missing")
    output_4 = client.out
    client.run("install -o boost/*:shared=True --build missing --build boost .")
    output_5 = client.out
    assert output_4 == output_5


def test_install_anonymous(client):
    # https://github.com/conan-io/conan/issues/4871
    client.save({"conanfile.py": GenConanfile("pkg", "0.1")})
    client.run("create . --user=lasote --channel=testing")
    client.run("upload * --confirm -r default")
    client2 = TestClient(servers=client.servers, inputs=[])
    client2.run("install --requires=pkg/0.1@lasote/testing")
    assert "pkg/0.1@lasote/testing: Package installed" in client2.out


@pytest.mark.artifactory_ready
def test_install_without_ref(client):
    client.save({"conanfile.py": GenConanfile("lib", "1.0")})
    client.run('create .')
    assert "lib/1.0: Package '{}' created".format(NO_SETTINGS_PACKAGE_ID) in client.out

    client.run('upload lib/1.0 -c -r default')
    assert "Uploading recipe 'lib/1.0" in client.out

    client.run('remove "*" -c')

    # This fails, Conan thinks this is a path
    client.run('install lib/1.0', assert_error=True)
    fake_path = os.path.join(client.current_folder, "lib", "1.0")
    assert "Conanfile not found at {}".format(fake_path) in client.out

    # Try this syntax to upload too
    client.run('install --requires=lib/1.0@')
    client.run('upload lib/1.0 -c -r default')


@pytest.mark.artifactory_ready
def test_install_disabled_remote(client):
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=pkg --version=0.1 --user=lasote --channel=testing")
    client.run("upload * --confirm -r default")
    client.run("remote disable default")
    client.run("install --requires=pkg/0.1@lasote/testing -r default", assert_error=True)
    assert "ERROR: Remote 'default' can't be found or is disabled" in client.out
    client.run("remote enable default")
    client.run("install --requires=pkg/0.1@lasote/testing -r default")
    client.run("remote disable default")
    client.run("install --requires=pkg/0.1@lasote/testing --update -r default", assert_error=True)
    assert "ERROR: Remote 'default' can't be found or is disabled" in client.out


def test_install_no_remotes(client):
    client.save({"conanfile.py": GenConanfile("pkg", "0.1")})
    client.run("create .")
    client.run("upload * --confirm -r default")
    client.run("remove * -c")
    client.run("install --requires=pkg/0.1 -nr", assert_error=True)
    assert "ERROR: Package 'pkg/0.1' not resolved: No remote defined" in client.out
    client.run("install --requires=pkg/0.1")  # this works without issue
    client.run("install --requires=pkg/0.1 -nr")  # and now this too, pkg in cache


def test_install_skip_disabled_remote():
    client = TestClient(servers=OrderedDict({"default": TestServer(),
                                             "server2": TestServer(),
                                             "server3": TestServer()}),
                        inputs=2*["admin", "password"])
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=pkg --version=0.1 --user=lasote --channel=testing")
    client.run("upload * --confirm -r default")
    client.run("upload * --confirm -r server3")
    client.run("remove * -c")
    client.run("remote disable default")
    client.run("install --requires=pkg/0.1@lasote/testing", assert_error=False)
    assert "Trying with 'default'..." not in client.out


def test_install_without_update_fail(client):
    # https://github.com/conan-io/conan/issues/9183
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=zlib --version=1.0")
    client.run("upload * --confirm -r default")
    client.save({"conanfile.py": GenConanfile().with_requires("zlib/1.0")})
    client.run("remote disable default")
    client.run("install .")
    assert "zlib/1.0: Already installed" in client.out


def test_install_version_range_reference(client):
    # https://github.com/conan-io/conan/issues/5905
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=pkg --version=0.1 --user=user --channel=channel")
    client.run("install --requires=pkg/[*]@user/channel")
    assert "pkg/0.1@user/channel: Already installed!" in client.out
    client.run("install --requires=pkg/[>0]@user/channel")
    assert "pkg/0.1@user/channel: Already installed!" in client.out


def test_install_error_never(client):
    client.save({"conanfile.py": GenConanfile("hello0", "0.1")})
    client.run("create .")
    client.run("install . --build never --build missing", assert_error=True)
    assert "ERROR: --build=never not compatible with other options" in client.out
    client.run("install conanfile.py --build never --build Hello", assert_error=True)
    assert "ERROR: --build=never not compatible with other options" in client.out
    client.run("install ./conanfile.py --build never --build outdated", assert_error=True)
    assert "ERROR: --build=never not compatible with other options" in client.out


def test_package_folder_available_consumer():
    """
    The package folder is not available when doing a consumer conan install "."
    We don't want to provide the package folder for the "cmake install" nor the "make install",
    as a consumer you could call the build system and pass the prefix PATH manually.
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.cmake import cmake_layout
    class HelloConan(ConanFile):
        settings = "os", "arch", "build_type"
        def layout(self):
            cmake_layout(self)
        def generate(self):
            self.output.warning("Package folder is None? {}".format(self.package_folder is None))
            self.output.warning("Package folder: {}".format(self.package_folder))
    """)
    client.save({"conanfile.py": conanfile})

    # Installing it with "install ." with output folder
    client.run("install . -of=my_build")
    assert "WARN: Package folder is None? True" in client.out

    # Installing it with "install ." without output folder
    client.run("install .")
    assert "WARN: Package folder is None? True" in client.out


def test_install_multiple_requires_cli():
    """
    Test that it is possible to install multiple --requires=xxx --requires=yyy
    """
    c = TestClient()
    c.save({"conanfile.py": GenConanfile()})
    c.run("create . --name=pkg1 --version=0.1")
    c.run("create . --name=pkg2 --version=0.1")

    c.run("graph info --requires=pkg1/0.1 --requires=pkg2/0.1")
    assert "pkg1/0.1" in c.out
    assert "pkg2/0.1" in c.out
    c.run("install --requires=pkg1/0.1 --requires=pkg2/0.1")
    assert "pkg1/0.1" in c.out
    assert "pkg2/0.1" in c.out
    c.run("lock create --requires=pkg1/0.1 --requires=pkg2/0.1 --lockfile-out=conan.lock")
    lock = c.load("conan.lock")
    assert "pkg1/0.1" in lock
    assert "pkg2/0.1" in lock


def test_install_json_formatter():
    """
    Tests the ``conan install . -f json`` result
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class MyTest(ConanFile):
            name = "pkg"
            version = "0.2"

            def package_info(self):
                self.cpp_info.libs = ["pkg"]
                self.cpp_info.includedirs = ["path/includes/pkg", "other/include/path/pkg"]
                self.cpp_info.libdirs = ["one/lib/path/pkg"]
                self.cpp_info.defines = ["pkg_onedefinition", "pkg_twodefinition"]
                self.cpp_info.cflags = ["pkg_a_c_flag"]
                self.cpp_info.cxxflags = ["pkg_a_cxx_flag"]
                self.cpp_info.sharedlinkflags = ["pkg_shared_link_flag"]
                self.cpp_info.exelinkflags = ["pkg_exe_link_flag"]
                self.cpp_info.sysroot = "/path/to/folder/pkg"
                self.cpp_info.frameworks = ["pkg_oneframework", "pkg_twoframework"]
                self.cpp_info.system_libs = ["pkg_onesystemlib", "pkg_twosystemlib"]
                self.cpp_info.frameworkdirs = ["one/framework/path/pkg"]
                self.cpp_info.set_property("pkg_config_name", "pkg_other_name")
                self.cpp_info.set_property("pkg_config_aliases", ["pkg_alias1", "pkg_alias2"])
                self.cpp_info.components["cmp1"].libs = ["libcmp1"]
                self.cpp_info.components["cmp1"].set_property("pkg_config_name", "compo1")
                self.cpp_info.components["cmp1"].set_property("pkg_config_aliases", ["compo1_alias"])
                self.cpp_info.components["cmp1"].sysroot = "/another/sysroot"
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create .")
    client.save({"conanfile.py": GenConanfile().with_name("hello").with_version("0.1")
                .with_require("pkg/0.2")}, clean_first=True)
    client.run("install . -f json")
    info = json.loads(client.stdout)
    nodes = info["graph"]["nodes"]
    hello_pkg_ref = 'hello/0.1'  # no revision available
    pkg_pkg_ref = 'pkg/0.2#926714b5fb0a994f47ec37e071eba1da'
    hello_cpp_info = pkg_cpp_info = None
    for _, n in nodes.items():
        ref = n["ref"]
        if ref == hello_pkg_ref:
            assert n['binary'] is None
            hello_cpp_info = n['cpp_info']
        elif ref == pkg_pkg_ref:
            assert n['binary'] == "Cache"
            pkg_cpp_info = n['cpp_info']

    hello = nodes["0"]
    assert hello["ref"] == hello_pkg_ref
    assert hello["recipe_folder"] == client.current_folder
    assert hello["build_folder"] == client.current_folder
    assert hello["generators_folder"] == client.current_folder
    assert hello["package_folder"] is None

    assert hello_cpp_info and pkg_cpp_info
    # hello/0.1 cpp_info
    assert hello_cpp_info['root']["libs"] is None
    assert len(hello_cpp_info['root']["bindirs"]) == 1
    assert len(hello_cpp_info['root']["libdirs"]) == 1
    assert hello_cpp_info['root']["sysroot"] is None
    assert hello_cpp_info['root']["properties"] is None
    # pkg/0.2 cpp_info
    # root info
    assert pkg_cpp_info['root']["libs"] == ['pkg']
    assert len(pkg_cpp_info['root']["bindirs"]) == 1
    assert len(pkg_cpp_info['root']["libdirs"]) == 1
    assert pkg_cpp_info['root']["sysroot"] == '/path/to/folder/pkg'
    assert pkg_cpp_info['root']["system_libs"] == ['pkg_onesystemlib', 'pkg_twosystemlib']
    assert pkg_cpp_info['root']['cflags'] == ['pkg_a_c_flag']
    assert pkg_cpp_info['root']['cxxflags'] == ['pkg_a_cxx_flag']
    assert pkg_cpp_info['root']['defines'] == ['pkg_onedefinition', 'pkg_twodefinition']
    assert pkg_cpp_info['root']["properties"] == {
        'pkg_config_aliases': ['pkg_alias1', 'pkg_alias2'],
        'pkg_config_name': 'pkg_other_name'}
    # component info
    assert pkg_cpp_info['cmp1']["libs"] == ['libcmp1']
    assert pkg_cpp_info['cmp1']["bindirs"][0].endswith("bin")  # Abs path /bin
    assert pkg_cpp_info['cmp1']["libdirs"][0].endswith("lib")  # Abs path /lib
    assert pkg_cpp_info['cmp1']["sysroot"] == "/another/sysroot"
    assert pkg_cpp_info['cmp1']["properties"] == {'pkg_config_aliases': ['compo1_alias'],
                                                  'pkg_config_name': 'compo1'}


def test_upload_skip_binaries_not_hit_server():
    """
    When upload_policy = "skip", no need to try to install from servers
    """
    c = TestClient(servers={"default": None})  # Broken server, will raise error if used
    conanfile = GenConanfile("pkg", "0.1").with_class_attribute('upload_policy = "skip"')
    c.save({"conanfile.py": conanfile})
    c.run("export .")
    c.run("install --requires=pkg/0.1 --build=missing")
    # This would crash if hits the server, but it doesnt
    assert "pkg/0.1: Created package" in c.out


def test_upload_skip_build_missing():
    c = TestClient(default_server_user=True)
    pkg1 = GenConanfile("pkg1", "1.0").with_class_attribute('upload_policy = "skip"')
    pkg2 = GenConanfile("pkg2", "1.0").with_requirement("pkg1/1.0", visible=False)
    pkg3 = GenConanfile("pkg3", "1.0").with_requirement("pkg2/1.0")
    c.save({"pkg1/conanfile.py": pkg1,
            "pkg2/conanfile.py": pkg2,
            "pkg3/conanfile.py": pkg3,
            })
    c.run("create pkg1")
    c.run("create pkg2")
    c.run("remove pkg1/*:* -c")  # remove binaries
    c.run("create pkg3 --build=missing")
    assert re.search(r"Skipped binaries(\s*)pkg1/1.0", c.out)


def test_upload_skip_build_compatibles():
    c = TestClient()
    pkg1 = textwrap.dedent("""\
        from conan import ConanFile
        class Pkg1(ConanFile):
            name = "pkg1"
            version = "1.0"
            settings = "build_type"
            upload_policy = "skip"
            def compatibility(self):
                if self.settings.build_type == "Debug":
                    return [{"settings": [("build_type", "Release")]}]
            """)
    pkg2 = GenConanfile("pkg2", "1.0").with_requirement("pkg1/1.0")
    pkg3 = GenConanfile("pkg3", "1.0").with_requirement("pkg2/1.0")
    c.save({"pkg1/conanfile.py": pkg1,
            "pkg2/conanfile.py": pkg2,
            "pkg3/conanfile.py": pkg3,
            })
    c.run("create pkg1 -s build_type=Release")
    pkg1id = c.created_package_id("pkg1/1.0")
    c.run("create pkg2 -s build_type=Release")
    c.run("remove pkg1/*:* -c")  # remove binaries
    c.run("create pkg3 -s build_type=Release --build=missing")
    c.assert_listed_binary({"pkg1/1.0": (pkg1id, "Build")})
    c.run("install pkg3 -s build_type=Debug --build=missing")
    c.assert_listed_binary({"pkg1/1.0": (pkg1id, "Cache")})


def test_install_json_format():
    # https://github.com/conan-io/conan/issues/14414
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class MyTest(ConanFile):
            name = "pkg"
            version = "0.1"
            settings = "build_type"

            def package_info(self):
                self.conf_info.define("user.myteam:myconf", "myvalue")
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create .")
    client.run("install --requires=pkg/0.1 --format=json")
    data = json.loads(client.stdout)
    conf_info = data["graph"]["nodes"]["1"]["conf_info"]
    assert {'user.myteam:myconf': 'myvalue'} == conf_info


def test_install_json_format_not_visible():
    """
    The dependencies that are needed at built time, even if they are not visible, or not standard
    libraries (headers=False, libs=False, run=False), like for example some build assets
    So direct dependencies of a consumer package or a package that needs to be built cannot be
    skipped
    """
    c = TestClient()
    dep = GenConanfile("dep", "0.1").with_package_file("somefile.txt", "contents!!!")
    app = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import load

        class Pkg(ConanFile):
            settings = "os", "arch", "build_type"
            name="pkg"
            version="0.0.1"

            def requirements(self):
                self.requires("dep/0.1", visible=False, headers=False, libs=False, run=False)

            def build(self):
                p = os.path.join(self.dependencies["dep"].package_folder, "somefile.txt")
                c = load(self, p)
                self.output.info(f"LOADED! {c}")
        """)
    c.save({"dep/conanfile.py": dep,
            "app/conanfile.py": app})
    c.run("export-pkg dep")
    c.run("install app --format=json")
    c.assert_listed_binary({"dep/0.1": ("da39a3ee5e6b4b0d3255bfef95601890afd80709", "Cache")})
    data = json.loads(c.stdout)
    pkg_folder = data["graph"]["nodes"]["1"]["package_folder"]
    assert pkg_folder is not None

    c.run("create app")
    assert "pkg/0.0.1: LOADED! contents!!!" in c.out



import os
import platform
import subprocess

import pytest

from conan.tools.files import replace_in_file, collect_libs
from conan.internal.model.cpp_info import CppInfo
from conan.internal.model.layout import Infos
from conan.test.utils.mocks import ConanFileMock, RedirectedTestOutput
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import redirect_output
from conan.test.utils.env import environment_update
from conan.internal.util.files import load, md5, save
from conan.internal.util.runners import check_output_runner


class RunnerMock:
    def __init__(self, return_ok=True, output=None):
        self.command_called = None
        self.return_ok = return_ok
        self.output = output

    def __call__(self, command):  # @UnusedVariable
        self.command_called = command
        return 0 if self.return_ok else 1


def test_replace_in_file():
    text = u'J\xe2nis\xa7'
    tmp_folder = temp_folder()

    win_file = os.path.join(tmp_folder, "win_encoding.txt")
    text = text.encode("Windows-1252", "ignore")
    with open(win_file, "wb") as handler:
        handler.write(text)
    replace_in_file(ConanFileMock(), win_file, "nis", "nus", encoding="Windows-1252")

    content = load(win_file, encoding="Windows-1252")
    assert "nis" not in  content
    assert "nus" in content


class TestToolsTest:

    def test_load_save(self):
        folder = temp_folder()
        path = os.path.join(folder, "file")
        save(path, u"äüïöñç")
        content = load(path)
        assert content == u"äüïöñç"

    def test_md5(self):
        result = md5(u"äüïöñç")
        assert "dfcc3d74aa447280a7ecfdb98da55174" == result

    def test_environment_nested(self):
        with environment_update({"A": "1", "Z": "40"}):
            with environment_update({"A": "1", "B": "2"}):
                with environment_update({"A": "2", "B": "2"}):
                    assert os.getenv("A") == "2"
                    assert os.getenv("B") == "2"
                    assert os.getenv("Z") == "40"
                assert os.getenv("A") == "1"
                assert os.getenv("B",) == "2"
            assert os.getenv("A",) == "1"
            assert os.getenv("Z") == "40"

        assert os.getenv("A") is None
        assert os.getenv("B") is None
        assert os.getenv("Z") is None

    def test_check_output_runner(self):
        payload = "hello world"
        output = check_output_runner("echo {}".format(payload), stderr=subprocess.STDOUT)
        assert payload in str(output)


class TestCollectLib:

    def test_collect_libs(self):
        output = RedirectedTestOutput()
        with (redirect_output(output)):
            conanfile = ConanFileMock()
            # Without package_folder
            result = collect_libs(conanfile)
            assert []  == result

            # Default behavior
            conanfile.folders.set_base_package(temp_folder())
            mylib_path = os.path.join(conanfile.package_folder, "lib", "mylib.lib")
            save(mylib_path, "")
            conanfile.cpp = Infos()
            result = collect_libs(conanfile)
            assert ["mylib"]  == result

            # Custom folder
            customlib_path = os.path.join(conanfile.package_folder, "custom_folder", "customlib.lib")
            save(customlib_path, "")
            result = collect_libs(conanfile, folder="custom_folder")
            assert ["customlib"]  == result

            # Custom folder doesn't exist
            result = collect_libs(conanfile, folder="fake_folder")
            assert []  == result
            assert "Lib folder doesn't exist, can't collect libraries:" in output.getvalue()
            output.clear()

            # Use cpp_info.libdirs
            conanfile.cpp_info.libdirs = ["lib", "custom_folder"]
            result = collect_libs(conanfile)
            assert ["customlib", "mylib"]  == result

            # Custom folder with multiple libdirs should only collect from custom folder
            assert ["lib", "custom_folder"] == conanfile.cpp_info.libdirs
            result = collect_libs(conanfile, folder="custom_folder")
            assert ["customlib"]  == result

            # Warn same lib different folders
            conanfile = ConanFileMock()
            conanfile.folders.set_base_package(temp_folder())
            conanfile.cpp = Infos()
            custom_mylib_path = os.path.join(conanfile.package_folder, "custom_folder", "mylib.lib")
            lib_mylib_path = os.path.join(conanfile.package_folder, "lib", "mylib.lib")
            save(custom_mylib_path, "")
            save(lib_mylib_path, "")
            conanfile.cpp_info.libdirs = ["lib", "custom_folder"]

            output.clear()
            result = collect_libs(conanfile)
            assert ["mylib"]  == result

            # Warn lib folder does not exist with correct result
            conanfile = ConanFileMock()
            conanfile.folders.set_base_package(temp_folder())
            conanfile.cpp = Infos()
            lib_mylib_path = os.path.join(conanfile.package_folder, "lib", "mylib.lib")
            save(lib_mylib_path, "")
            no_folder_path = os.path.join(conanfile.package_folder, "no_folder")
            conanfile.cpp_info.libdirs = ["no_folder", "lib"]  # 'no_folder' does NOT exist
            output.clear()
            result = collect_libs(conanfile)
            assert ["mylib"]  == result
            assert ("WARN: Lib folder doesn't exist, "
                    "can't collect libraries: %s") % no_folder_path in output.getvalue()

    @pytest.mark.skipif(platform.system() == "Windows", reason="Needs symlinks support")
    def test_collect_libs_symlinks(self):
        # Keep only the shortest lib name per group of symlinks
        conanfile = ConanFileMock()
        conanfile.folders.set_base_package(temp_folder())
        conanfile.cpp_info = CppInfo(set_defaults=True)
        version_mylib_path = os.path.join(conanfile.package_folder, "lib", "libmylib.1.0.0.dylib")
        soversion_mylib_path = os.path.join(conanfile.package_folder, "lib", "libmylib.1.dylib")
        lib_mylib_path = os.path.join(conanfile.package_folder, "lib", "libmylib.dylib")
        lib_mylib2_path = os.path.join(conanfile.package_folder, "lib", "libmylib.2.dylib")
        lib_mylib3_path = os.path.join(conanfile.package_folder, "custom_folder", "libmylib.3.dylib")
        save(version_mylib_path, "")
        os.symlink(version_mylib_path, soversion_mylib_path)
        os.symlink(soversion_mylib_path, lib_mylib_path)
        save(lib_mylib2_path, "")
        save(lib_mylib3_path, "")
        conanfile.cpp_info.libdirs = ["lib", "custom_folder"]
        result = collect_libs(conanfile)
        assert ["mylib", "mylib.2", "mylib.3"]  == result

    def test_self_collect_libs(self):
        output = RedirectedTestOutput()
        with redirect_output(output):
            conanfile = ConanFileMock()
            # Without package_folder
            result = collect_libs(conanfile)
            assert []  == result

            # Default behavior
            conanfile.folders.set_base_package(temp_folder())
            mylib_path = os.path.join(conanfile.package_folder, "lib", "mylib.lib")
            save(mylib_path, "")
            conanfile.cpp = Infos()
            result = collect_libs(conanfile)
            assert ["mylib"]  == result

            # Custom folder
            customlib_path = os.path.join(conanfile.package_folder, "custom_folder", "customlib.lib")
            save(customlib_path, "")
            result = collect_libs(conanfile, folder="custom_folder")
            assert ["customlib"]  == result

            # Custom folder doesn't exist
            output.clear()
            result = collect_libs(conanfile, folder="fake_folder")
            assert []  == result
            assert "Lib folder doesn't exist, can't collect libraries:" in output.getvalue()

            # Use cpp_info.libdirs
            conanfile.cpp_info.libdirs = ["lib", "custom_folder"]
            result = collect_libs(conanfile)
            assert ["customlib", "mylib"]  == result

            # Custom folder with multiple libdirs should only collect from custom folder
            assert ["lib", "custom_folder"] == conanfile.cpp_info.libdirs
            result = collect_libs(conanfile, folder="custom_folder")
            assert ["customlib"]  == result

            # Warn same lib different folders
            conanfile = ConanFileMock()
            conanfile.folders.set_base_package(temp_folder())
            conanfile.cpp = Infos()
            custom_mylib_path = os.path.join(conanfile.package_folder, "custom_folder", "mylib.lib")
            lib_mylib_path = os.path.join(conanfile.package_folder, "lib", "mylib.lib")
            save(custom_mylib_path, "")
            save(lib_mylib_path, "")
            conanfile.cpp_info.libdirs = ["lib", "custom_folder"]
            output.clear()
            result = collect_libs(conanfile)
            assert ["mylib"]  == result

            # Warn lib folder does not exist with correct result
            conanfile = ConanFileMock()
            conanfile.folders.set_base_package(temp_folder())
            conanfile.cpp = Infos()
            lib_mylib_path = os.path.join(conanfile.package_folder, "lib", "mylib.lib")
            save(lib_mylib_path, "")
            no_folder_path = os.path.join(conanfile.package_folder, "no_folder")
            conanfile.cpp_info.libdirs = ["no_folder", "lib"]  # 'no_folder' does NOT exist
            output.clear()
            result = collect_libs(conanfile)
            assert ["mylib"]  == result
            assert ("WARN: Lib folder doesn't exist, "
                    "can't collect libraries: %s" % no_folder_path in output.getvalue())

    @pytest.mark.skipif(platform.system() == "Windows", reason="Needs symlinks support")
    def test_self_collect_libs_symlinks(self):
        # Keep only the shortest lib name per group of symlinks
        conanfile = ConanFileMock()
        conanfile.folders.set_base_package(temp_folder())
        conanfile.cpp_info = CppInfo(set_defaults=True)
        version_mylib_path = os.path.join(conanfile.package_folder, "lib", "libmylib.1.0.0.dylib")
        soversion_mylib_path = os.path.join(conanfile.package_folder, "lib", "libmylib.1.dylib")
        lib_mylib_path = os.path.join(conanfile.package_folder, "lib", "libmylib.dylib")
        lib_mylib2_path = os.path.join(conanfile.package_folder, "lib", "libmylib.2.dylib")
        lib_mylib3_path = os.path.join(conanfile.package_folder, "custom_folder", "libmylib.3.dylib")
        save(version_mylib_path, "")
        os.symlink(version_mylib_path, soversion_mylib_path)
        os.symlink(soversion_mylib_path, lib_mylib_path)
        save(lib_mylib2_path, "")
        save(lib_mylib3_path, "")
        conanfile.cpp_info.libdirs = ["lib", "custom_folder"]
        result = collect_libs(conanfile)
        assert ["mylib", "mylib.2", "mylib.3"]  == result



import os
import platform

import pytest

from conan.internal.model.manifest import FileTreeManifest
from conan.test.utils.test_files import temp_folder
from conan.internal.util.files import load, md5, save


@pytest.mark.skipif(platform.system() == "Windows", reason="decent symlinks only")
def test_symlinks_manifest():
    """ The symlinks are coded in the manifest, the key is the path of the symlink and the hash
        is the hash of the contents or empty string if this is a broken symlink"""
    tmp_dir = temp_folder()
    save(os.path.join(tmp_dir, "foo.txt"), "bar")
    os.symlink("foo.txt", os.path.join(tmp_dir, "foofake.txt"))
    os.symlink("foo_missing.txt", os.path.join(tmp_dir, "var.txt"))
    manifest = FileTreeManifest.create(tmp_dir)

    manifest.save(tmp_dir)
    read_manifest = FileTreeManifest.load(tmp_dir)
    assert read_manifest.file_sums == {'foo.txt': '37b51d194a7513e45b56f6524f2d51f2',
                                       'foofake.txt': '4fd8cc85ca9eebd2fa3c550069ce2846',
                                       'var.txt': '6cc1cda38d517f2e5e47a803a343d17d'}


def test_tree_manifest():
    tmp_dir = temp_folder()
    files = {"one.ext": "aalakjshdlkjahsdlkjahsdljkhsadljkhasljkdhlkjashd",
             "path/to/two.txt": "asdas13123",
             "two.txt": "asdasdasdasdasdasd",
             "folder/damn.pyc": "binarythings",
             "folder/damn.pyo": "binarythings2",
             "pythonfile.pyc": "binarythings3"}
    for filename, content in files.items():
        save(os.path.join(tmp_dir, filename), content)

    manifest = FileTreeManifest.create(tmp_dir)

    manifest.save(tmp_dir)
    readed_manifest = FileTreeManifest.load(tmp_dir)

    assert readed_manifest.time == manifest.time
    assert readed_manifest == manifest
    # Not included the pycs or pyo
    assert set(manifest.file_sums.keys()) == {'folder/damn.pyc', 'folder/damn.pyo', 'one.ext',
                                              'path/to/two.txt', 'pythonfile.pyc', 'two.txt'}

    for filepath, md5readed in manifest.file_sums.items():
        content = files[filepath]
        assert md5(content), md5readed


def test_already_pyc_in_manifest():
    tmp_dir = temp_folder()
    save(os.path.join(tmp_dir, "man.txt"), "1478122267\nconanfile.pyc: "
                                           "2bcac725a0e6843ef351f4d18cf867ec\n"
                                           "conanfile.py: 2bcac725a0e6843ef351f4d18cf867ec\n"
                                           "conanfile.pyo: 2bcac725a0e6843ef351f4d18cf867ec\n")

    read_manifest = FileTreeManifest.loads(load(os.path.join(tmp_dir, "man.txt")))
    # Not included the pycs or pyo
    assert set(read_manifest.file_sums.keys()) == {"conanfile.py", "conanfile.pyc", "conanfile.pyo"}


def test_special_chars():
    tmp_dir = temp_folder()
    save(os.path.join(tmp_dir, "conanmanifest.txt"), "1478122267\nsome: file.py: 123\n")
    read_manifest = FileTreeManifest.load(tmp_dir)
    assert read_manifest.file_sums["some: file.py"] == "123"


def test_pycache_included():
    tmp_dir = temp_folder()
    files = {"__pycache__/damn.py": "binarythings",
             "pythonfile.pyc": "binarythings3"}
    for filename, content in files.items():
        save(os.path.join(tmp_dir, filename), content)

    manifest = FileTreeManifest.create(tmp_dir)
    manifest = repr(manifest)
    assert "pythonfile.pyc" in manifest
    assert "__pycache__/damn.py" in manifest



import os
import platform
import textwrap

import pytest

from conan.tools.files import replace_in_file
from conan.test.utils.mocks import ConanFileMock

from conan.test.utils.tools import TestClient

new_value = "will_break_next"


@pytest.mark.skipif(platform.system() != "Linux", reason="No OS specific test")
@pytest.mark.tool("cmake")
def test_cpp_info_sources():
    c = TestClient()
    c.run("new cmake_lib -d name=hello -d version=1.0")
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import copy

        class HelloConan(ConanFile):
            name = "hello"
            version = "1.0"
            exports_sources = "src/*", "include/*"
            package_type = "header-library"

            def package(self):
                copy(self, "*.h", self.source_folder, self.package_folder)
                copy(self, "*.cpp", self.source_folder, self.package_folder)

            def package_info(self):
                self.cpp_info.sources = ["src/hello.cpp"]
    """)
    c.save({"conanfile.py": conanfile})
    # Check that the hello library builds in test_package
    c.run(f"create . -c tools.cmake.cmakedeps:new={new_value}")
    # Check content of the generated files
    c.run(f"install --requires=hello/1.0 -g=CMakeConfigDeps "
          f"-c tools.cmake.cmakedeps:new={new_value}")
    cmake = c.load("hello-Targets-release.cmake")
    assert "add_library(hello::hello INTERFACE IMPORTED)" in cmake
    assert "set_property(TARGET hello::hello APPEND PROPERTY INTERFACE_SOURCES\n"\
           "             $<$<CONFIG:RELEASE>:${hello_PACKAGE_FOLDER_RELEASE}/src/hello.cpp>)" in cmake


@pytest.mark.skipif(platform.system() != "Linux", reason="No OS specific test")
@pytest.mark.tool("cmake")
def test_cpp_info_component_sources():
    c = TestClient()
    c.run("new cmake_lib -d name=hello -d version=1.0")
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import copy

        class HelloConan(ConanFile):
            name = "hello"
            version = "1.0"
            exports_sources = "src/*", "include/*"
            package_type = "header-library"

            def package(self):
                copy(self, "*.h", self.source_folder, self.package_folder)
                copy(self, "*.cpp", self.source_folder, self.package_folder)

            def package_info(self):
                self.cpp_info.components["my_comp"].sources = ["src/hello.cpp", "src/other.cpp"]
    """)
    c.save({
        "conanfile.py": conanfile,
        "src/other.cpp": "",
    })

    # Make test_package link with component's target
    test_package_cmakelists_path = os.path.join(c.current_folder, "test_package", "CMakeLists.txt")
    replace_in_file(ConanFileMock(), test_package_cmakelists_path, "hello::hello", "hello::my_comp")
    test_package_cmakelists_content = c.load(test_package_cmakelists_path)
    assert "target_link_libraries(example hello::my_comp)" in test_package_cmakelists_content
    # Check that the hello library builds in test_package
    c.run(f"create . -c tools.cmake.cmakedeps:new={new_value}")
    # Check the content of the generated files
    c.run(f"install --requires=hello/1.0 -g=CMakeConfigDeps "
          f"-c tools.cmake.cmakedeps:new={new_value}")
    cmake = c.load("hello-Targets-release.cmake")
    assert "add_library(hello::hello INTERFACE IMPORTED)" in cmake
    assert "add_library(hello::my_comp INTERFACE IMPORTED)" in cmake
    assert "set_property(TARGET hello::my_comp APPEND PROPERTY INTERFACE_SOURCES\n"\
           "             $<$<CONFIG:RELEASE>:${hello_PACKAGE_FOLDER_RELEASE}/src/hello.cpp"\
           " ${hello_PACKAGE_FOLDER_RELEASE}/src/other.cpp>)" in cmake



import os
import textwrap

from conan.internal.model.manifest import FileTreeManifest
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save


def test_called_before_digest():
    """ Test that 'post_export' hook is called before computing the digest of the
        exported folders
    """
    t = TestClient()
    complete_hook = textwrap.dedent("""\
        import os
        from conan.tools.files import save
        def post_export(conanfile):
            save(conanfile, os.path.join(conanfile.export_folder, "myfile.txt"), "content!!")
        """)
    hook_path = os.path.join(t.paths.hooks_path, "complete_hook", "hook_complete.py")
    save(hook_path, complete_hook)
    t.save({'conanfile.py': GenConanfile("pkg", "0.1")})
    t.run("export .")
    ref_layout = t.exported_layout()
    manifest = FileTreeManifest.load(ref_layout.export())
    assert "myfile.txt" in manifest.file_sums



import textwrap

from conan.internal.model.conan_file import ConanFile
from conan.test.utils.tools import TestClient


class TestConanFile:
    def test_conanfile_naming(self):
        for member in vars(ConanFile):
            if member.startswith('_') and not member.startswith("__"):
                assert member.startswith('_conan')

        conanfile = ConanFile(None)

        for member in vars(conanfile):
            if member.startswith('_') and not member.startswith("__"):
                assert member.startswith('_conan')

    def test_conanfile_naming_complete(self):
        client = TestClient()
        conanfile = """from conan import ConanFile
class Pkg(ConanFile):
    pass
    def package_info(self):
        for member in Pkg.__dict__:
            if member.startswith('_') and not member.startswith("__"):
                assert(member.startswith('_conan'))
        for member in vars(self):
            if member.startswith('_') and not member.startswith("__"):
                assert(member.startswith('_conan'))
"""
        client.save({"conanfile.py": conanfile})
        client.run("create . --name=pkga --version=0.1 --user=user --channel=testing")
        client.save({"conanfile.py": conanfile.replace("pass",
                                                       "requires = 'pkga/0.1@user/testing'")})
        client.run("create . --name=pkgb --version=0.1 --user=user --channel=testing")
        client.save({"conanfile.py": conanfile.replace("pass",
                                                       "requires = 'pkgb/0.1@user/testing'")})
        client.run("create . --name=pkgc --version=0.1 --user=user --channel=testing")

    def test_conanfile_new_print(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            import sys
            class Pkg(ConanFile):
                def source(self):
                    print("Test1", file=sys.stderr)
                    print("Test2")
            """)
        client.save({"conanfile.py": conanfile})
        client.run("source .")
        assert "Test1" in client.stderr
        assert "Test2" in client.stderr
        assert "" == client.stdout

    def test_conanfile_new_print_save(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            import sys
            class Pkg(ConanFile):
                def source(self):
                    with open("myfile.txt", "w") as f:
                        print("Test1", file=f)
            """)
        client.save({"conanfile.py": conanfile})
        client.run("source .")
        assert "Test1" in client.load("myfile.txt")



from conan.internal.api.detect.detect_api import default_cstd
from conan.internal.model.version import Version
from conan.test.utils.mocks import ConanFileMock, MockSettings
from conan.tools.build.flags import cstd_flag


def _make_cstd_flag(compiler, compiler_version, cstd=None):
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({"compiler": compiler,
                                       "compiler.version": compiler_version,
                                       "compiler.cstd": cstd})
    return cstd_flag(conanfile)


def test_gcc_cstd_flags():
    assert _make_cstd_flag("gcc", "4.2", "99") == "-std=c99"
    assert _make_cstd_flag("clang", "4.2", "99") == "-std=c99"
    assert _make_cstd_flag("apple-clang", "4.2", "99") == "-std=c99"
    assert _make_cstd_flag("msvc", "192", "11") == "/std:c11"
    assert _make_cstd_flag("msvc", "192", "17") == "/std:c17"
    assert _make_cstd_flag("msvc", "191", "11") is None
    assert _make_cstd_flag("msvc", "191", "17") is None


def _make_cstd_default(compiler, compiler_version):
    return default_cstd(compiler, Version(compiler_version))


def test_gcc_cstd_defaults():
    assert _make_cstd_default("gcc", "4") == "gnu99"
    assert _make_cstd_default("gcc", "5") == "gnu11"
    assert _make_cstd_default("gcc", "6") == "gnu11"
    assert _make_cstd_default("gcc", "6.1") == "gnu11"
    assert _make_cstd_default("gcc", "7.3") == "gnu11"
    assert _make_cstd_default("gcc", "8.1") == "gnu17"
    assert _make_cstd_default("gcc", "11") == "gnu17"
    assert _make_cstd_default("gcc", "11.1") == "gnu17"
    assert _make_cstd_default("gcc", "15.1") == "gnu23"


def test_clang_cstd_defaults():
    assert _make_cstd_default("clang", "2") == "gnu99"
    assert _make_cstd_default("clang", "2.1") == "gnu99"
    assert _make_cstd_default("clang", "3.0") == "gnu99"
    assert _make_cstd_default("clang", "3.1") == "gnu99"
    assert _make_cstd_default("clang", "3.4") == "gnu99"
    assert _make_cstd_default("clang", "3.5") == "gnu99"
    assert _make_cstd_default("clang", "5") == "gnu11"
    assert _make_cstd_default("clang", "5.1") == "gnu11"
    assert _make_cstd_default("clang", "6") == "gnu11"
    assert _make_cstd_default("clang", "7") == "gnu11"
    assert _make_cstd_default("clang", "8") == "gnu11"
    assert _make_cstd_default("clang", "9") == "gnu11"
    assert _make_cstd_default("clang", "10") == "gnu11"
    assert _make_cstd_default("clang", "11") == "gnu17"
    assert _make_cstd_default("clang", "12") == "gnu17"
    assert _make_cstd_default("clang", "13") == "gnu17"
    assert _make_cstd_default("clang", "14") == "gnu17"
    assert _make_cstd_default("clang", "15") == "gnu17"
    assert _make_cstd_default("clang", "16") == "gnu17"
    assert _make_cstd_default("clang", "17") == "gnu17"
    assert _make_cstd_default("clang", "18") == "gnu17"
    assert _make_cstd_default("clang", "19") == "gnu17"
    assert _make_cstd_default("clang", "20"), "gnu17"


def test_apple_clang_cppstd_defaults():
    assert _make_cstd_default("apple-clang", "9") == "gnu99"
    assert _make_cstd_default("apple-clang", "10") == "gnu11"
    assert _make_cstd_default("apple-clang", "11") == "gnu11"
    assert _make_cstd_default("apple-clang", "12") == "gnu17"
    assert _make_cstd_default("apple-clang", "13") == "gnu17"
    assert _make_cstd_default("apple-clang", "14") == "gnu17"
    assert _make_cstd_default("apple-clang", "15") == "gnu17"



from requests import Response

from conan.test.utils.tools import TestClient, TestRequester


conanfile = """
from conan import ConanFile
from conan.tools.files import download

class HelloConan(ConanFile):
    def source(self):
        download(self, "http://foo", "bar.txt")
"""


class MyRequester(TestRequester):

    def get(self, _, **kwargs):
        print("TIMEOUT: {}".format(kwargs.get("timeout", "NOT SPECIFIED")))
        resp = Response()
        resp.status_code = 200
        resp._content = b''
        return resp


class TestRequester:

    def test_requester_timeout(self):
        client = TestClient(requester_class=MyRequester)
        client.save_home({"global.conf": "core.net.http:timeout=4.3"})
        client.save({"conanfile.py": conanfile})
        client.run("create . --name=foo --version=1.0")
        assert "TIMEOUT: 4.3" in client.out

    def test_requester_timeout_tuple(self):
        client = TestClient(requester_class=MyRequester)
        client.save_home({"global.conf": "core.net.http:timeout=(2, 3.4)"})
        client.save({"conanfile.py": conanfile})
        client.run("create . --name=foo --version=1.0")
        assert "TIMEOUT: (2, 3.4)" in client.out

    def test_request_infinite_timeout(self):
        # Test that not having timeout works
        client = TestClient(requester_class=MyRequester)
        client.save_home({"global.conf": "core.net.http:timeout=-1"})
        client.save({"conanfile.py": conanfile})
        client.run("create . --name=foo --version=1.0")
        assert "TIMEOUT: NOT SPECIFIED" in client.out

    def test_unset_request_timeout_use_default_one(self):
        client = TestClient(requester_class=MyRequester)
        client.save_home({"global.conf": "core.net.http:timeout=!"})
        client.save({"conanfile.py": conanfile})
        client.run("create . --name=foo --version=1.0")
        assert "TIMEOUT: (30, 60)" in client.out



from math import floor

import time

from conans.server.revision_list import RevisionList


class TestRevisionList:

    def test_remove_latest(self):
        rev = RevisionList()
        rev.add_revision("rev1")
        rev.add_revision("rev2")

        dumped = rev.dumps()
        loaded = RevisionList.loads(dumped)
        assert rev == loaded
        assert loaded.latest_revision().revision == "rev2"

        loaded.remove_revision("rev2")
        assert loaded.latest_revision().revision == "rev1"

    def test_remove_non_latest(self):
        rev = RevisionList()
        rev.add_revision("rev1")
        rev.add_revision("rev2")

        dumped = rev.dumps()
        loaded = RevisionList.loads(dumped)
        loaded.remove_revision("rev1")
        assert loaded.latest_revision().revision == "rev2"

    def test_compatibility_with_timestamps(self):
        the_time = floor(time.time())
        old_contents = '{"revisions": [{"revision": "rev1", "time": %s}, ' \
                       '{"revision": "rev2", "time": %s}]}' % (the_time, the_time)
        r_list = RevisionList.loads(old_contents)
        when = r_list.get_time("rev1")
        assert when == the_time



import os
import pytest

from conan.internal.api.uploader import compress_files
from conan.internal.paths import PACKAGE_TGZ_NAME
from conan.test.utils.test_files import temp_folder
from conan.internal.util.files import save


class TestRemoteManager:

    def test_compress_files(self):
        folder = temp_folder()
        save(os.path.join(folder, "one_file.txt"), "The contents")
        save(os.path.join(folder, "Two_file.txt"), "Two contents")

        files = {
            "one_file.txt": os.path.join(folder, "one_file.txt"),
            "Two_file.txt": os.path.join(folder, "Two_file.txt"),
        }

        path = compress_files(files, PACKAGE_TGZ_NAME, dest_dir=folder)
        assert os.path.exists(path)
        expected_path = os.path.join(folder, PACKAGE_TGZ_NAME)
        assert path == expected_path



import fnmatch
import os
import re
import textwrap
from xml.dom import minidom

from jinja2 import Template

from conan.internal import check_duplicated_generator
from conan.errors import ConanException
from conan.internal.api.install.generators import relativize_path
from conan.internal.model.dependencies import get_transitive_requires
from conan.tools.microsoft.visual import msvc_platform_from_arch
from conan.internal.util.files import load, save

VALID_LIB_EXTENSIONS = (".so", ".lib", ".a", ".dylib", ".bc")


class MSBuildDeps:
    """
    MSBuildDeps class generator
    conandeps.props: unconditional import of all *direct* dependencies only
    """

    _vars_props = textwrap.dedent("""\
        <?xml version="1.0" encoding="utf-8"?>
        <Project ToolsVersion="4.0" xmlns="http://schemas.microsoft.com/developer/msbuild/2003">
          <PropertyGroup Label="ConanVariables">
            <Conan{{name}}RootFolder>{{root_folder}}</Conan{{name}}RootFolder>
            <Conan{{name}}BinaryDirectories>{{bin_dirs}}</Conan{{name}}BinaryDirectories>
            {% if host_context %}
            <Conan{{name}}CompilerFlags>{{compiler_flags}}</Conan{{name}}CompilerFlags>
            <Conan{{name}}LinkerFlags>{{linker_flags}}</Conan{{name}}LinkerFlags>
            <Conan{{name}}PreprocessorDefinitions>{{definitions}}</Conan{{name}}PreprocessorDefinitions>
            <Conan{{name}}IncludeDirectories>{{include_dirs}}</Conan{{name}}IncludeDirectories>
            <Conan{{name}}ResourceDirectories>{{res_dirs}}</Conan{{name}}ResourceDirectories>
            <Conan{{name}}LibraryDirectories>{{lib_dirs}}</Conan{{name}}LibraryDirectories>
            <Conan{{name}}Libraries>{{libs}}</Conan{{name}}Libraries>
            <Conan{{name}}SystemLibs>{{system_libs}}</Conan{{name}}SystemLibs>
            {% endif %}
          </PropertyGroup>
        </Project>
        """)

    _conf_props = textwrap.dedent("""\
        <?xml version="1.0" encoding="utf-8"?>
        <Project ToolsVersion="4.0" xmlns="http://schemas.microsoft.com/developer/msbuild/2003">
          <ImportGroup Label="PropertySheets">
            {% for dep in deps %}
            <Import Condition="'$(conan_{{dep}}_props_imported)' != 'True'" Project="conan_{{dep}}.props"/>
            {% endfor %}
          </ImportGroup>
          <ImportGroup Label="PropertySheets">
            <Import Project="{{vars_filename}}"/>
          </ImportGroup>
          {% if host_context %}
          <PropertyGroup>
            <ConanDebugPath>$(Conan{{name}}BinaryDirectories);$(ConanDebugPath)</ConanDebugPath>
            <LocalDebuggerEnvironment>PATH=$(ConanDebugPath);%PATH%</LocalDebuggerEnvironment>
            <DebuggerFlavor>WindowsLocalDebugger</DebuggerFlavor>
            {% if ca_exclude %}
            <CAExcludePath>$(Conan{{name}}IncludeDirectories);$(CAExcludePath)</CAExcludePath>
            {% endif %}
          </PropertyGroup>
          <ItemDefinitionGroup>
            <ClCompile>
              <AdditionalIncludeDirectories>$(Conan{{name}}IncludeDirectories)%(AdditionalIncludeDirectories)</AdditionalIncludeDirectories>
              <PreprocessorDefinitions>$(Conan{{name}}PreprocessorDefinitions)%(PreprocessorDefinitions)</PreprocessorDefinitions>
              <AdditionalOptions>$(Conan{{name}}CompilerFlags) %(AdditionalOptions)</AdditionalOptions>
            </ClCompile>
            <Link>
              <AdditionalLibraryDirectories>$(Conan{{name}}LibraryDirectories)%(AdditionalLibraryDirectories)</AdditionalLibraryDirectories>
              <AdditionalDependencies>$(Conan{{name}}Libraries)%(AdditionalDependencies)</AdditionalDependencies>
              <AdditionalDependencies>$(Conan{{name}}SystemLibs)%(AdditionalDependencies)</AdditionalDependencies>
              <AdditionalOptions>$(Conan{{name}}LinkerFlags) %(AdditionalOptions)</AdditionalOptions>
            </Link>
            <Midl>
              <AdditionalIncludeDirectories>$(Conan{{name}}IncludeDirectories)%(AdditionalIncludeDirectories)</AdditionalIncludeDirectories>
            </Midl>
            <ResourceCompile>
              <AdditionalIncludeDirectories>$(Conan{{name}}IncludeDirectories)%(AdditionalIncludeDirectories)</AdditionalIncludeDirectories>
              <PreprocessorDefinitions>$(Conan{{name}}PreprocessorDefinitions)%(PreprocessorDefinitions)</PreprocessorDefinitions>
            </ResourceCompile>
          </ItemDefinitionGroup>
          {% else %}
          <PropertyGroup>
            <ExecutablePath>$(Conan{{name}}BinaryDirectories)$(ExecutablePath)</ExecutablePath>
          </PropertyGroup>
          {% endif %}
        </Project>
        """)

    def __init__(self, conanfile):
        """
        :param conanfile: ``< ConanFile object >`` The current recipe object. Always use ``self``.
        """
        self._conanfile = conanfile
        #: Defines the build type. By default, the value of ``settings.build_type``.
        self.configuration = conanfile.settings.build_type
        #: Defines the configuration key used to conditionally select which property sheet to
        #: import (defaults to ``"Configuration"``).
        self.configuration_key = "Configuration"
        # TODO: This platform is not exactly the same as ``msbuild_arch``, because it differs
        # in x86=>Win32
        #: Platform name, e.g., ``Win32`` if ``settings.arch == "x86"``.
        self.platform = msvc_platform_from_arch(str(conanfile.settings.arch))
        #: Defines the platform key used to conditionally select which property sheet to
        #: import (defaults to ``"Platform"``).
        self.platform_key = "Platform"
        ca_exclude = "tools.microsoft.msbuilddeps:exclude_code_analysis"
        #: List of packages names patterns to add Visual Studio ``CAExcludePath`` property
        #: to each match as part of its ``conan_[DEP]_[CONFIG].props``. By default, value given by
        #: ``tools.microsoft.msbuilddeps:exclude_code_analysis`` configuration.
        self.exclude_code_analysis = self._conanfile.conf.get(ca_exclude, check_type=list)

    def generate(self):
        """
        Generates ``conan_<pkg>_<config>_vars.props``, ``conan_<pkg>_<config>.props``,
        and ``conan_<pkg>.props`` files into the ``conanfile.generators_folder``.
        """
        check_duplicated_generator(self, self._conanfile)
        if self.configuration is None:
            raise ConanException("MSBuildDeps.configuration is None, it should have a value")
        if self.platform is None:
            raise ConanException("MSBuildDeps.platform is None, it should have a value")
        generator_files = self._content()
        for generator_file, content in generator_files.items():
            save(generator_file, content)

    def _config_filename(self):
        props = [self.configuration,
                 self.platform]
        name = "".join("_%s" % v for v in props)
        return name.lower()

    def _condition(self):
        props = [(self.configuration_key, self.configuration),
                 (self.platform_key, self.platform)]
        condition = " And ".join("'$(%s)' == '%s'" % (k, v) for k, v in props)
        return condition

    @staticmethod
    def _dep_name(dep, build):
        dep_name = dep.ref.name
        if build:  # dep.context == CONTEXT_BUILD:
            dep_name += "_build"
        return MSBuildDeps._get_valid_xml_format(dep_name)

    @staticmethod
    def _get_valid_xml_format(name):
        return re.compile(r"[.+]").sub("_", name)

    def _vars_props_file(self, require, dep, name, cpp_info, build):
        """
        content for conan_vars_poco_x86_release.props, containing the variables for 1 config
        This will be for 1 package or for one component of a package
        :return: varfile content
        """

        def add_valid_ext(libname, libdirs=None):
            ext = os.path.splitext(libname)[1]
            if ext in VALID_LIB_EXTENSIONS:
                return f"{libname};"

            lib_name = f"{libname}.lib"
            if libdirs and not any(lib_name in os.listdir(d) for d in libdirs if os.path.isdir(d)):
                meson_name = f"lib{libname}.a"
                if any(meson_name in os.listdir(d) for d in libdirs if os.path.isdir(d)):
                    lib_name = meson_name
            return f"{lib_name};"

        pkg_placeholder = "$(Conan{}RootFolder)".format(name)

        def escape_path(path):
            # https://docs.microsoft.com/en-us/visualstudio/msbuild/
            #                          how-to-escape-special-characters-in-msbuild
            # https://docs.microsoft.com/en-us/visualstudio/msbuild/msbuild-special-characters
            return path.lstrip("/")

        def join_paths(paths):
            # TODO: ALmost copied from CMakeDeps TargetDataContext
            ret = []
            for p in paths:
                assert os.path.isabs(p), "{} is not absolute".format(p)
                full_path = escape_path(p)
                if full_path.startswith(root_folder):
                    rel = full_path[len(root_folder)+1:]
                    full_path = ("%s/%s" % (pkg_placeholder, rel))
                ret.append(full_path)
            return "".join("{};".format(e) for e in ret)

        root_folder = dep.recipe_folder if dep.package_folder is None else dep.package_folder
        root_folder = escape_path(root_folder)
        # Make the root_folder relative to the generated conan_vars_xxx.props file
        relative_root_folder = relativize_path(root_folder, self._conanfile,
                                               "$(MSBuildThisFileDirectory)", normalize=False)

        bin_dirs = join_paths(cpp_info.bindirs)
        res_dirs = join_paths(cpp_info.resdirs)
        include_dirs = join_paths(cpp_info.includedirs)
        lib_dirs = join_paths(cpp_info.libdirs)
        libs = "".join([add_valid_ext(lib, cpp_info.libdirs) for lib in cpp_info.libs])
        # TODO: Missing objects
        system_libs = "".join([add_valid_ext(sys_dep) for sys_dep in cpp_info.system_libs])
        definitions = "".join("%s;" % d for d in cpp_info.defines)
        compiler_flags = " ".join(cpp_info.cxxflags + cpp_info.cflags)
        linker_flags = " ".join(cpp_info.sharedlinkflags + cpp_info.exelinkflags)

        # traits logic
        if require and not require.headers:
            include_dirs = ""
        if require and not require.libs:
            lib_dirs = ""
            libs = ""
        if require and not require.libs and not require.headers:
            definitions = ""
            compiler_flags = ""
            linker_flags = ""
        if require and not require.run:
            bin_dirs = ""

        fields = {
            'name': name,
            'root_folder': relative_root_folder,
            'bin_dirs': bin_dirs,
            'res_dirs': res_dirs,
            'include_dirs': include_dirs,
            'lib_dirs': lib_dirs,
            'libs': libs,
            # TODO: Missing objects
            'system_libs': system_libs,
            'definitions': definitions,
            'compiler_flags': compiler_flags,
            'linker_flags': linker_flags,
            'host_context': not build
        }
        formatted_template = Template(self._vars_props, trim_blocks=True,
                                      lstrip_blocks=True).render(**fields)
        return formatted_template

    def _activate_props_file(self, dep_name, vars_filename, deps, build):
        """
        Actual activation of the VS variables, per configuration
            - conan_pkgname_x86_release.props / conan_pkgname_compname_x86_release.props
        :param dep_name: pkgname / pkgname_compname
        :param deps: the name of other things to be included: [dep1, dep2:compA, ...]
        :param build: if it is a build require or not
        """

        # TODO: This must include somehow the user/channel, most likely pattern to exclude/include
        # Probably also the negation pattern, exclude all not @mycompany/*
        ca_exclude = any(fnmatch.fnmatch(dep_name, p) for p in self.exclude_code_analysis or ())
        template = Template(self._conf_props, trim_blocks=True, lstrip_blocks=True)
        content_multi = template.render(host_context=not build, name=dep_name, ca_exclude=ca_exclude,
                                        vars_filename=vars_filename, deps=deps)
        return content_multi

    @staticmethod
    def _dep_props_file(dep_name, filename, aggregated_filename, condition, content=None):
        """
        The file aggregating all configurations for a given pkg / component
            - conan_pkgname.props
        """
        # Current directory is the generators_folder
        if content:
            content_multi = content  # Useful for aggregating multiple components in one pass
        elif os.path.isfile(filename):
            content_multi = load(filename)
        else:
            content_multi = textwrap.dedent("""\
            <?xml version="1.0" encoding="utf-8"?>
            <Project ToolsVersion="4.0" xmlns="http://schemas.microsoft.com/developer/msbuild/2003">
              <ImportGroup Label="PropertySheets">
              </ImportGroup>
              <PropertyGroup>
                <conan_{{name}}_props_imported>True</conan_{{name}}_props_imported>
              </PropertyGroup>
            </Project>
            """)
            content_multi = Template(content_multi).render({"name": dep_name})
        # parse the multi_file and add new import statement if needed
        dom = minidom.parseString(content_multi)
        import_vars = dom.getElementsByTagName('ImportGroup')[0]

        # Current vars
        children = import_vars.getElementsByTagName("Import")
        for node in children:
            if aggregated_filename == node.getAttribute("Project") \
                    and condition == node.getAttribute("Condition"):
                break
        else:  # create a new import statement
            import_node = dom.createElement('Import')
            import_node.setAttribute('Condition', condition)
            import_node.setAttribute('Project', aggregated_filename)
            import_vars.appendChild(import_node)

        content_multi = dom.toprettyxml()
        content_multi = "\n".join(line for line in content_multi.splitlines() if line.strip())
        return content_multi

    def _conandeps(self):
        """ this is a .props file including direct declared dependencies
        """
        # Current directory is the generators_folder
        conandeps_filename = "conandeps.props"
        direct_deps = self._conanfile.dependencies.filter({"direct": True})
        pkg_aggregated_content = textwrap.dedent("""\
            <?xml version="1.0" encoding="utf-8"?>
            <Project ToolsVersion="4.0" xmlns="http://schemas.microsoft.com/developer/msbuild/2003">
              <ImportGroup Label="PropertySheets">
              </ImportGroup>
            </Project>
            """)
        for req, dep in direct_deps.items():
            dep_name = self._dep_name(dep, req.build)
            filename = "conan_%s.props" % dep_name
            comp_condition = "'$(conan_%s_props_imported)' != 'True'" % dep_name
            pkg_aggregated_content = self._dep_props_file("", conandeps_filename, filename,
                                                          condition=comp_condition,
                                                          content=pkg_aggregated_content)
        return {conandeps_filename: pkg_aggregated_content}

    def _package_props_files(self, require, dep, build=False):
        """ all the files for a given package:
        - conan_pkgname_vars_config.props: definition of variables, one per config
        - conan_pkgname_config.props: The one using those variables. This is very different for
                                      Host and build, build only activate <ExecutablePath>
        - conan_pkgname.props: Conditional aggregate xxx_config.props based on active config
        """
        conf_name = self._config_filename()
        condition = self._condition()
        dep_name = self._dep_name(dep, build)
        result = {}
        pkg_deps = get_transitive_requires(self._conanfile, dep)  # only non-skipped dependencies
        if dep.cpp_info.has_components:
            pkg_aggregated_content = None
            for comp_name, comp_info in dep.cpp_info.components.items():
                full_comp_name = "{}_{}".format(dep_name, self._get_valid_xml_format(comp_name))
                vars_filename = "conan_%s_vars%s.props" % (full_comp_name, conf_name)
                activate_filename = "conan_%s%s.props" % (full_comp_name, conf_name)
                comp_filename = "conan_%s.props" % full_comp_name
                pkg_filename = "conan_%s.props" % dep_name

                public_deps = []  # To store the xml dependencies/file names
                for required_pkg, required_comp in comp_info.parsed_requires():
                    if required_pkg is not None:  # Points to a component of a different package
                        try:
                            required = pkg_deps[required_pkg]
                        except KeyError:  # The transitive dep might have been skipped
                            required = None
                        if required:  # The transitive dep might have been skipped
                            required_name = required.ref.name
                            public_deps.append(required_name if required_pkg == required_comp
                                               else "{}_{}".format(required_name, required_comp))
                    else:  # Points to a component of same package
                        public_deps.append("{}_{}".format(dep_name, required_comp))
                public_deps = [self._get_valid_xml_format(d) for d in public_deps]
                result[vars_filename] = self._vars_props_file(require, dep, full_comp_name,
                                                              comp_info, build=build)
                result[activate_filename] = self._activate_props_file(full_comp_name, vars_filename,
                                                                      public_deps, build=build)
                result[comp_filename] = self._dep_props_file(full_comp_name, comp_filename,
                                                             activate_filename, condition)
                comp_condition = "'$(conan_%s_props_imported)' != 'True'" % full_comp_name
                pkg_aggregated_content = self._dep_props_file(dep_name, pkg_filename, comp_filename,
                                                              condition=comp_condition,
                                                              content=pkg_aggregated_content)
                result[pkg_filename] = pkg_aggregated_content
        else:
            cpp_info = dep.cpp_info
            vars_filename = "conan_%s_vars%s.props" % (dep_name, conf_name)
            activate_filename = "conan_%s%s.props" % (dep_name, conf_name)
            pkg_filename = "conan_%s.props" % dep_name
            public_deps = [self._dep_name(d, build) for d in pkg_deps.values()]

            result[vars_filename] = self._vars_props_file(require, dep, dep_name, cpp_info,
                                                          build=build)
            result[activate_filename] = self._activate_props_file(dep_name, vars_filename,
                                                                  public_deps, build=build)
            result[pkg_filename] = self._dep_props_file(dep_name, pkg_filename, activate_filename,
                                                        condition=condition)
        return result

    def _content(self):
        if not self._conanfile.settings.get_safe("build_type"):
            raise ConanException("The 'msbuild' generator requires a 'build_type' setting value")
        result = {}

        for req, dep in self._conanfile.dependencies.host.items():
            result.update(self._package_props_files(req, dep, build=False))
        for req, dep in self._conanfile.dependencies.test.items():
            result.update(self._package_props_files(req, dep, build=False))
        for req, dep in self._conanfile.dependencies.build.items():
            result.update(self._package_props_files(req, dep, build=True))

        # Include all direct build_requires for host context. This might change
        result.update(self._conandeps())

        return result
